      subroutine tau_head
C+------------------------------------------------------------------------------
C
C   ---------------
C   T A U _ H E A D
C   ---------------
C
C   Description
C   -----------
C   Sets some useful TAURUS info in the .MORE structure of a file.
C
C
C   Scope of program
C   ----------------
C   - Data size irrelevant
C   - Data type irrelevant
C   - Magic values irrelevant
C   - Quality and error arrays irrelevant
C   - Batch execution supported.
C
C
C   Environment
C   -----------
C   FIGARO
C
C
C   Parameters (read from VARS.DST)
C   -------------------------------
C   CALFILE     Name of TAUCAL file (character)(prompted for)
C
C   IMAGE       Name of file to which the header info is to be written
C               (character)(prompted for)
C
C   RUNNUM      Run number where info can be found in TAUCAL. (integer)
C               (prompted for).
C
C   WHICH_MASK  Number of the MASK already defined in TAUCAL to use.
C               (integer)(prompted for)
C
C   WHICH_RVF   Number of the RVF already defined in TAUCAL to use.
C               (integer)(prompted for)
C
C
C   Keywords (read from VARS.DST)
C   -----------------------------
C   USE_TAU   Flag for use of TAUCAL file. (prompted for)
C
C
C   Parameters (read from or written to TAUCAL file)
C   ------------------------------------------------
C
C   Parameters which reside in .RUN structures have the run type given in
C   parentheses.  Those without a run type reside in the top level of the
C   calibration structure.
C
C   (ANY)              FSR         Free spectral range in MPUs. (real)(read)
C
C   (ANY)              XC          The X position of the optical axis.
C                                  (real)(read)
C
C   (ANY)              YC          The Y position of the optical axis.
C                                  (real)(read)
C
C   (ANY)              DMPU        Z axis increment in MPUs/frame (real)(read)
C
C   (ANY)              ALAM0       Wavelength zeropoint in microns.
C                                 (real)(read)
C
C   (ANY)              DLAM        Dispersion in microns/pixel (real)(read)
C
C   .MASK              NMASK       Number of MASKs already defined in TAUCAL.
C                                  (integer)(read)
C
C   .MASK.MASKn        PIXSIZE_X   Pixel size as defined by MASK. (real)(read)
C
C   .CAMERA            FOCLEN      Focal length of the camera used. (real)(read)
C
C   .CAMERA            FOCNUM      F number of camera used (real)(read)
C
C   .COLLIMATOR        FOCLEN      Focal length of collimator used. (real)(read)
C
C   .TELESCOPE         SCALE       Scale in arcsec/mm of the telescope
C                                  (real)(read)
C
C   .TELESCOPE         NAME        Name of the telescope used. (character)(read)
C
C   .TAURUS            VERSION     Version of TAURUS used in observations
C                                  (integer)(read)
C
C   .RVF               NRVF        Number of RVFs already defined in TAUCAL.
C                                  (integer)(read)
C
C   .RVF.RVFn          GAP         Etalon gap in microns. (real)(read)
C
C   .MORE.TAURUS       ALAM0       Wavelength zeropoint in angstroms. (real)
C                                  (written)
C
C   .MORE.TAURUS       DLAM        Dispersion in angstroms/pixel (real)(written)
C
C   .MORE.TAURUS       FSR         Free spectral range in angstroms. (real)
C                                  (written)
C
C   .MORE.TAURUS       XCEN        X position of optical axis. (real)(written)
C
C   .MORE.TAURUS       YCEN        Y position of optical axis. (real)(written)
C
C   .MORE.TAURUS       GAP         Etalon gap in microns. (real)(written)
C
C   .MORE.TAURUS       PIXSIZE     Pixel size in microns (real)(written)
C
C   .MORE.TAURUS       SCALE       Scale in arcsec/pixel. (real)(written)
C
C   .MORE.TAURUS       TELESCOPE   Name of the telescope used. (character)
C                                  (written)
C
C   .MORE.TAURUS       CAM_FNUM    F number of camera used. (real)(written)
C
C   .MORE.TAURUS       TAU_VER     Version of TAURUS used. (real)(written)
C
C
C   Propagation of data structure
C   -----------------------------
C   Above information is written to .MORE.TAURUS structure. No other
C   changes are made to the data structure.
C
C
C   Method
C   ------
C   - The IMAGE structure is opened and the .MORE.TAURUS sub-structure is
C     added to it.
C   - If TAUCAL is being used, then the information is read from there,
C     otherwise the user is prompted for all sorts of info.
C   - Info is written to header.
C
C
C   External functions & subroutines called
C   ---------------------------------------
C   Library DSA:
C     DSA_ADD_STRUCTURE
C     DSA_CLOSE
C     DSA_NAMED_INPUT
C     DSA_OPEN
C     DSA_READ_STRUCT_DEF
C     DSA_WRFLUSH
C     DSA_WRUSER
C
C   Library DTA:
C     DTA_CRNAM
C     DTA_CRVAR
C     DTA_DLVAR
C     DTA_STRUC
C
C   Library ICH:
C     ICH_LEN
C
C   Library NDP:
C     NDP_GET_IMAGE_INFO
C
C   Library PAR:
C     PAR_RDCHAR
C     PAR_RDVAL
C
C   Library TAU:
C     TAU_CAL_DTA_NAME
C     TAU_CAL_FIND_STRUCT
C     TAU_CAL_OPEN
C     TAU_CAL_RDCHR
C     TAU_CAL_RDNUM
C     TAU_SETCHR
C     TAU_SETNUM
C
C
C   Internal subroutines called
C   ---------------------------
C   None
C
C
C   INCLUDE statements
C   ------------------
C   INCLUDE 'DTACODES'
C   INCLUDE 'NDP_NUMERIC_RANGES
C
C
C   Extensions to FORTRAN77
C   -----------------------
C   IMPLICIT NONE / INCLUDE / Names > 6 characters /
C
C
C   Possible future upgrades
C   ------------------------
C
C
C   Author/s
C   --------
C   Jim Lewis    RGO  (jrl@uk.ac.cam.ast.mail)
C
C
C   History
C   -------
C   30-NOV-90   - Original program
C   19-FEB-91   - Fixed so that it doesn't fall over if there is no .MORE
C                 structure. (JRL)
C   03-JUL-91   - Small bug fixed which could have affected ability to
C                 create .MORE structure. (JRL)
C   05-JAN-91   - Removed call to DSA_FIND_REF. (JRL)
C   06-JAN-91   - Added calls to TAU_SETNUM and TAU_SETCHR. (JRL)
C   31-JAN-94   - Modified for use with standalone UNIX FIGARO v4.0 (JRL)
C
C
C   Other Comments
C   --------------
C
C+------------------------------------------------------------------------------
      implicit none
C
C     Functions
C
      integer   ich_len
C
C     Local variables
C
      real          alam0       ! Wavelength zeropoint
      logical       badpix      ! Flag for magic values in main data array
      real          cam_fnum    ! Camera f number
      real          cam_foc     ! Camera focal length
      real          col_foc     ! Collimator focal length
      real          dlam        ! Wavelength increment
      real          dmpu        ! Increment in MPUs/frame
      character     dta_name*80 ! DTA name of input file
      integer       dta_status  ! DTA status variable
      real          dummy       ! Real dummy variable
      real          fsr         ! Free spectral range
      real          gap         ! Etalon gap
      character     image*80    ! Name of image
      integer       length      ! Length of DTA_NAME
      integer       len_ml      ! Length of mask label
      integer       len_rv      ! Length of RVF label
      integer       lname       ! Length of NAME
      character     masklab*80  ! TAUCAL mask label
      character     name*80     ! DTA structure name
      real          pixsize_x   ! Pixel size
      integer       runnum      ! Run number of file to be changed
      character     rvflab*80   ! TAUCAL RVF label
      real          scale       ! Telescope scale
      integer       status      ! DSA and TAUCAL status variable
      logical       struct      ! Flag for object's being a structure or not.
      logical       tauexist    ! Flag for existence of TAUCAL
      integer       tau_ver     ! Version of TAURUS
      character     tel_name*80 ! Name of telescope
      character     type*10     ! Data type of main array
      real          xc          ! X position of optical axis
      real          yc          ! Y position of optical axis
C
C     Numeric ranges
C
      include 'NDP_NUMERIC_RANGES'
C
C     DTA error codes
C
      include 'DTACODES'
C
C     Get started
C
      status = 0
      call dsa_open(status)
      if (status .ne. 0) go to 500
C
C     Open TAUCAL file.
C
      call tau_cal_open('cal','calfile',tauexist,status)
      if (status .ne. 0) go to 500
C
C     Get file to be with header to be changed
C
      call par_rdchar('image',' ',image)
      call dsa_named_input('image',image,status)
      call ndp_get_image_info('image',.true.,.false.,type,badpix,
     : status)
C
C     Need to check for the existence of .MORE.TAURUS structure.  First
C     get the DTA names right...
C
      call tau_cal_dta_name('image',image,dta_name,length,status)
      if (status .ne. 0) go to 500
C
C     First check for .MORE substructure...
C
      dta_status = 0
      call dta_crnam(dta_name(:length),'MORE',0,0,name,dta_status)
      dta_status = 0
      call dta_struc(name,struct,dta_status)
C
C     If the .MORE doesn't exist the create it.  If it does exist but
C     isn't a structure, then delete it and recreate it.
C
      if (dta_status .eq. dta_notfnd) then
         call dsa_wruser('Couldn''t find .MORE structure, so I''ll')
         call dsa_wruser(' try and create it for you')
         call dsa_wrflush
         dta_status = 0
         call dta_crvar(name,'struct',dta_status)
         if (dta_status .ne. 0) then
            call dsa_wruser('Couldn''t create .MORE structure --')
            call dsa_wruser(' bye bye...')
            call dsa_wrflush
            go to 500
         end if
      else if (.not. struct) then
         call dsa_wruser('I''ve found the .MORE variable, but it')
         call dsa_wruser(' doesn''t appear to be a structure. I''ll')
         call dsa_wruser(' try to delete it and create a new one')
         call dsa_wrflush
         dta_status = 0
         call dta_dlvar(name,dta_status)
         if (dta_status .ne. 0) then
            call dsa_wruser('Well, I tried, but I can''t delete it --')
            call dsa_wruser(' bye bye...')
            call dsa_wrflush
            go to 500
         end if
         call dta_crvar(name,'struct',dta_status)
         if (dta_status .ne. 0) then
            call dsa_wruser('Couldn''t create new .MORE structure --')
            call dsa_wruser(' bye bye...')
            call dsa_wrflush
            go to 500
         end if
      end if
C
C     Now, check and see if the .MORE.TAURUS structure already exists.
C
      dta_status = 0
      call dta_crnam(dta_name(:length),'MORE.TAURUS',0,0,name,
     : dta_status)
      lname = ich_len(name)
      dta_status = 0
      call dta_struc(name,struct,dta_status)
C
C     If .MORE.TAURUS structure exists, but isn't a structure, then
C     delete it.
C
      if ((dta_status .eq. 0) .and. (.not. struct)) then
         call dsa_wruser('I''ve found the .MORE.TAURUS variable, but')
         call dsa_wruser(' it doesn''t appear to be a structure. I''ll')
         call dsa_wruser(' try to delete it and create a new one')
         call dsa_wrflush
         dta_status = 0
         call dta_dlvar(name,dta_status)
         if (dta_status .ne. 0) then
            call dsa_wruser('Well, I tried, but I can''t delete it --')
            call dsa_wruser(' bye bye...')
            call dsa_wrflush
            go to 500
         end if
      else if (dta_status .ne. 0) then
         struct = .false.
         dta_status = 0
      end if
C
C     If required, read header .DEF file and create .TAURUS sub-structure
C
      if (.not. struct) then       
         call dsa_read_struct_def('TAU_HEAD_SUB.DEF',status)
         call dsa_add_structure('image','tau_head_sub',
     :   'head_sub_struct',status)
         if (status .ne. 0) go to 500
      end if
C
C     Get run number for file
C
      if (tauexist) then
         call par_rdval('runnum',float(min_int),float(max_int),1.0,' ',
     :   dummy)
         runnum = nint(dummy)
      end if
C
C     Get info from .run structure
C
      call tau_cal_rdnum('cal',runnum,'alam0','alam0',tauexist,
     : .true.,min_float,max_float,'microns',1,alam0,status)
      alam0 = alam0*1.0e4
      call tau_cal_rdnum('cal',runnum,'dlam','dlam',tauexist,
     : .true.,min_float,max_float,'microns/frame',1,dlam,status)
      dlam = dlam*1.0e4
      call tau_cal_rdnum('cal',runnum,'fsr','fsr',tauexist,
     : .true.,min_float,max_float,'MPUs',1,fsr,status)
      call tau_cal_rdnum('cal',runnum,'dmpu','dmpu',tauexist,
     : .true.,min_float,max_float,'MPUs/frame',1,dmpu,status)
      fsr = fsr*dlam/dmpu
      call tau_cal_rdnum('cal',runnum,'xc','xcen',tauexist,
     : .true.,min_float,max_float,'pixels',1,xc,status)
      call tau_cal_rdnum('cal',runnum,'yc','ycen',tauexist,
     : .true.,min_float,max_float,'pixels',1,yc,status)
      if (status .ne. 0) go to 500
C
C     Write this stuff to the header now.  
C
      call tau_setnum(name,lname,'alam0',1,1,1,alam0,status)
      call tau_setnum(name,lname,'dlam',1,1,1,dlam,status)
      call tau_setnum(name,lname,'fsr',1,1,1,fsr,status)
      call tau_setnum(name,lname,'xcen',1,1,1,xc,status)
      call tau_setnum(name,lname,'ycen',1,1,1,yc,status)
      if (status .ne. 0) go to 500
C
C     Now get info from various other places in the TAUCAL file...
C
      call tau_cal_find_struct('cal','rvf','rvf',tauexist,rvflab,len_rv,
     : status)
      call tau_cal_rdnum('cal',0,rvflab(:len_rv)//'.gap','gap',
     : tauexist,.true.,min_float,max_float,'microns',1,gap,status)
      call tau_cal_find_struct('cal','mask','mask',tauexist,masklab,
     : len_ml,status)
      call tau_cal_rdnum('cal',0,masklab(:len_ml)//'.pixsize_x',
     : 'pixsize_x',tauexist,.true.,min_float,max_float,'microns',
     : 1,pixsize_x,status)
      call tau_cal_rdnum('cal',0,'camera.foclen','camera_focus',
     : tauexist,.true.,min_float,max_float,'mm',1,cam_foc,
     : status)
      call tau_cal_rdnum('cal',0,'collimator.foclen','col_focus',
     : tauexist,.true.,min_float,max_float,'mm',1,col_foc,
     : status)
      call tau_cal_rdnum('cal',0,'telescope.scale','tel_scale',
     : tauexist,.true.,min_float,max_float,'arcsec/mm',1,scale,
     : status)
      scale = scale*col_foc*pixsize_x/(1.0e3*cam_foc)
      call tau_cal_rdchr('cal',0,'telescope.name','tel_name',
     : tauexist,.true.,80,tel_name,status)
      call tau_cal_rdnum('cal',0,'camera.focnum','cam_fnum',
     : tauexist,.true.,min_float,max_float,' ',1,cam_fnum,status)
      call tau_cal_rdnum('cal',0,'taurus.version','tau_ver',
     : tauexist,.true.,float(min_int),float(max_int),' ',1,dummy,
     : status)
      tau_ver = nint(dummy)
      if (status .ne. 0) go to 500
C
C     Now write this stuff into the header...
C
      call tau_setnum(name,lname,'gap',1,1,1,gap,status)
      call tau_setnum(name,lname,'pixsize',1,1,1,pixsize_x,status)
      call tau_setnum(name,lname,'scale',1,1,1,scale,status)
      call tau_setchr(name,lname,'telescope',ich_len(tel_name),
     : tel_name,status)
      call tau_setnum(name,lname,'cam_fnum',1,1,1,cam_fnum,status)
      call tau_setnum(name,lname,'tau_ver',1,1,1,float(tau_ver),
     : status)
C
C     Tidy and exit
C
  500 call dsa_close(status)
      end
