      subroutine tau_cal_check_run(ref_name,runnum,type,taucal,status)
C+-----------------------------------------------------------------------------
C
C   ---------------------------------
C   T A U _ C A L _ C H E C K _ R U N
C   ---------------------------------
C
C   Description
C   -----------
C   Checks the run number given to make sure it's the right type.
C
C
C   Parameters (">" Input, "<" Output, "W" Workspace, "!" Changed)
C
C   (>) REF_NAME:  (character) Reference name for the TAUCAL file
C   (>) RUNNUM:    (integer) Run number as supplied by the user.
C   (>) TYPE:      (character) The type of run required.
C   (>) TAUCAL:    (logical) Flag for the existence of TAUCAL file
C   (!) STATUS:    (integer) Status flag.
C
C
C   Prompts
C   -------
C   None
C
C
C   External functions & subroutines called
C   ---------------------------------------
C   Library DSA:
C     DSA_WRFLUSH
C     DSA_WRUSER
C
C   Library DTA:
C     DTA_ERROR
C     DTA_RDVARC
C
C   Library ICH:
C     ICH_CI
C     ICH_FOLD
C     ICH_LEN
C
C
C   INCLUDE statements
C   ------------------
C   INCLUDE 'TAU_COMMON'
C                                                
C
C   Extensions to FORTRAN77
C   -----------------------
C   IMPLICIT NONE / Names > 6 characters
C
C
C   SUN-specific statements
C   -----------------------
C   None.
C
C
C   Author/s
C   --------
C   Jim Lewis    RGO  (jrl@uk.ac.cam.ast.mail)
C
C
C   History
C   -------
C   03-JUL-1990   - Original subroutine
C   03-JAN-1992   - Included TAUCAL common block. (JRL)
C   28-JAN-1994   - Modified for use with standalone UNIX FIGARO v4.0 (JRL)
C
C
C+-----------------------------------------------------------------------------
      implicit none
C
C     Parameters
C
      integer runnum,status
      character type*(*),ref_name*(*)
      logical taucal
C
C     Functions
C
      character ich_ci*8
      integer ich_fold
      integer ich_len
C
C     Local variables
C
      integer dumint,ldta,dta_status
      character number*8,dta_run*80,obstype*8,message*80,
     : typeup*8
C
C     TAUCAL common block
C
      include 'TAU_COMMON'
C
C     Inherited status
C
      if (status .ne. 0) go to 500
C
C     Return if not using a TAUCAL file
C
      if (.not. taucal) go to 500
C
C     Check run corresponding to runnum is the correct type...
C
      dta_status = 0
      number = ich_ci(runnum)
      dta_run = dta_name(:lendta)//'.run'//number(:ich_len(number))
      ldta = ich_len(dta_run)
      call dta_rdvarc(dta_run(:ldta)//'.type',8,obstype,dta_status)
      if (dta_status .ne. 0) go to 500
      dumint = ich_fold(obstype)
      typeup = type
      dumint = ich_fold(typeup)
      if (obstype .ne. typeup) then
         call dsa_wruser('Run number '//number(:ich_len(number))//
     :   ' is not of type '//typeup)
         call dsa_wrflush
         status = 1
         go to 500
      end if
C
C     Look for DTA errors too
C
  500 if (dta_status .ne. 0) then
         call dta_error(dta_status,message)
         call dsa_wruser(message(:ich_len(message)))
         call dsa_wrflush
         status = 1
      end if
      end
