      subroutine tau_cal_dta_name(ref_name,tau_cal_filename,dta_name,
     : lendta,status)
C+-----------------------------------------------------------------------------
C
C   -------------------------------
C   T A U _ C A L _ D T A _ N A M E
C   -------------------------------
C
C   Description
C   -----------
C   Create DTA object name of TAUCAL file.
C
C
C   Parameters (">" Input, "<" Output, "W" Workspace, "!" Changed)
C
C   (>) REF_NAME:         (character) Reference name of TAUCAL file.
C   (>) TAU_CAL_FILENAME: (character) Name of TAUCAL file
C   (<) DTA_NAME:         (character) DTA object name
C   (<) LENDTA:           (integer) Length of DTA object name
C   (!) STATUS:           (integer) Status flag
C
C
C   Prompts
C   -------
C   None
C
C
C   External functions & subroutines called
C   ---------------------------------------
C   Library DSA:
C     DSA_WRFLUSH
C     DSA_WRUSER
C
C   Library ICH:
C     ICH_DELIM
C     ICH_FOLD
C     ICH_LEN
C     ICH_VERIF
C
C
C   INCLUDE statements
C   ------------------
C   None
C                                                
C
C   Extensions to FORTRAN77
C   -----------------------
C   IMPLICIT NONE / Names > 6 characters 
C
C
C   SUN-specific statements
C   -----------------------
C   None.
C
C
C   Author/s
C   --------
C   Jim Lewis    RGO  (jrl@uk.ac.cam.ast.mail)
C
C
C   History
C   -------
C   03-JAN-1992   - Original version (JRL)
C   28-JAN-1994   - Modified for use with standalone UNIX FIGARO v4.0 (JRL)
C   11-OCT-1994   - Fixed the way that directories are found (JRL)
C
C
C+-----------------------------------------------------------------------------
      implicit none
C
C     Parameters
C
      character ref_name*(*),tau_cal_filename*(*),dta_name*(*)
      integer lendta,status
C
C     Functions
C
      integer ich_delim
      integer ich_fold
      integer ich_len
      integer ich_verif
C
C     Local variables
C
      integer ist,itest1,itest2,dumint,length
      character bigtest*65,struct*80,ref_name_uc*80,struct_uc*80
C
C     String for test
C
      data bigtest(1:26),bigtest(27:52),bigtest(53:65) 
     : /'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ',
     : '0123456789._-'/
C
C     Inherited status...
C
      if (status .ne. 0) go to 500
C
C     Create the DTA object name for the file...First look for a
C     parenthesis denoting extension and version number...
C
      ist = 1
      length = ich_len(tau_cal_filename)
      itest1 = ich_delim(tau_cal_filename,1,')')
      if (itest1 .ne. 0) then
         ist = itest1 + 1
C
C     Look for syntax denoting a directory
C
      else
         itest2 = ich_delim(tau_cal_filename,1,'/')
         do while (itest2 .ne. 0)
            ist = itest2 + 1
            itest2 = ich_delim(tau_cal_filename,ist,'/')
         end do
      end if
C
C     Now we know where to start looking, make sure that
C     no other naughty characters exist in the file name.
C
      itest1 = ich_verif(tau_cal_filename(:length),ist,bigtest)
      if (itest1 .ne. 0) then
         call dsa_wruser(' ')
         call dsa_wrflush
         call dsa_wruser('*****Error in TAUCAL file name*****')
         call dsa_wrflush
         call dsa_wruser(' ')
         call dsa_wrflush
         status = 1
         go to 500
      end if
C
C     Now find the first dot and construct the name
C
      itest1 = ich_delim(tau_cal_filename,ist,'.')
      if (itest1 .ne. 0) then
         struct = tau_cal_filename(itest1+1:ich_len(tau_cal_filename))
      else
         struct = ' '
      end if
      ref_name_uc = ref_name
      dumint = ich_fold(ref_name_uc)
      struct_uc = struct
      dumint = ich_fold(struct_uc)
      dta_name = ref_name_uc(:ich_len(ref_name_uc))//
     : struct_uc(:ich_len(struct_uc))
      lendta = ich_len(dta_name)
C
C     Exit
C
  500 continue
      end
                         
