      subroutine tau_cal_find_struct(ref,env,object,taucal,label,length,
     : status)
C+-----------------------------------------------------------------------------
C
C   -------------------------------------
C   T A U _ C A L _ F I N D _ S T R U C T
C   -------------------------------------
C
C   Description
C   -----------
C   Finds which TAUCAL sub-structure the user wants to use.  It
C   first checks how many sub-structures there are.  If there are
C   none, then an error occurs.  If there is more than one then
C   the user is prompted for the number of the sub-structure to
C   be used.  Output is a label for the TAUCAL sub-structure for
C   other TAUCAL routines to use.
C
C
C   Parameters (">" Input, "<" Output, "W" Workspace, "!" Changed)
C
C   (>) REF:       (character) Reference name for the TAUCAL file
C   (>) ENV:       (character) The environment in the TAUCAL file
C                              within which the object is nested.
C   (>) OBJECT:    (character) Name of the sub-structure desired
C   (>) TAUCAL:    (logical) Flag for the existence of TAUCAL file
C   (<) LABEL:     (character) Label for sub-structure.
C   (<) LENGTH:    (integer) Length of LABEL
C   (!) STATUS:    (integer) Status flag.
C
C
C   Prompts
C   -------
C   WHICH_"object" -- If there are more than one sub-structure, then
C                     this asks you which one you want.  The prompt
C                     is the word WHICH_ with the object name appended.
C
C
C   External functions & subroutines called
C   ---------------------------------------
C   Library DSA:
C     DSA_WRFLUSH
C     DSA_WRUSER
C
C   Library ICH:
C     ICH_CI
C     ICH_FOLD
C     ICH_LEN
C
C   Library PAR:
C     PAR_RDVAL
C
C   Library TAU:
C     TAU_CAL_RDNUM
C
C
C   INCLUDE statements
C   ------------------
C   INCLUDE 'NDP_NUMERIC_RANGES'
C                                                
C
C   Extensions to FORTRAN77
C   -----------------------
C   IMPLICIT NONE / INCLUDE / Names > 6 characters
C
C
C   SUN-specific statements
C   -----------------------
C   None.
C
C
C   Author/s
C   --------
C   Jim Lewis    RGO  (jrl@uk.ac.cam.ast.mail)
C
C
C   History
C   -------
C   19-NOV-1990   - Original subroutine
C   28-JAN-1994   - Modified for use with standalone UNIX FIGARO v4.0 (JRL)
C
C
C+-----------------------------------------------------------------------------
      implicit none
C
C     Parameters
C
      integer length,status
      logical taucal
      character ref*(*),env*(*),object*(*),label*(*)
C
C     Functions
C
      character ich_ci*8
      integer ich_fold
      integer ich_len
C
C     Local variables
C
      integer n,in,dumint
      real dummy
      character obj_up*80,message*80,an*1,env_up*80
C
C     Numeric ranges
C
      include 'NDP_NUMERIC_RANGES'
C
C     Inherited status
C
      if (status .ne. 0) go to 500
C
C     Don't bother if no TAUCAL file exists...
C
      if (.not. taucal) go to 500
C
C     Find the number of sub-structures in the environment
C
      call tau_cal_rdnum(ref,0,env//'.n'//object,' ',taucal,.false.,
     : 0.0,float(max_int),' ',1,dummy,status)
      if (status .ne. 0) go to 500
      n = nint(dummy)
C
C     Convert the environment and object to upper case
C
      env_up = env
      obj_up = object
      dumint = ich_fold(env_up)
      dumint = ich_fold(obj_up)
C
C     If there are no sub-structures, then quit
C
      if (n .eq. 0) then
         write(message,100) obj_up(:ich_len(obj_up)),
     :   env_up(:ich_len(env_up))
  100    format('No ',a,' sub-structures found in ',a)
         call dsa_wruser(message(:ich_len(message)))
         call dsa_wrflush
         status = 1
         go to 500
      end if
C
C     If there is only one sub-structure, then that's obviously
C     the one you want...
C
      if (n .eq. 1) then
         an = '1'
C
C     Otherwise, find the one you want...
C
      else
         write(message,101) n,obj_up(:ich_len(obj_up))
  101    format('You have a choice of',i2,1x,a,'s')
         call dsa_wruser(message(:ich_len(message)))
         call dsa_wrflush
         call par_rdval('which_'//object,1.0,float(n),1.0,' ',dummy)
         in = nint(dummy)
         an = ich_ci(in)
      end if
C
C     Set up a label for the substructure for future use...
C
      label = env(:ich_len(env))//'.'//object(:ich_len(object))//an
      length = ich_len(label)
C
C     Exit
C
  500 end
