      subroutine tau_cal_open(ref_name,par_name,tau_exist,status)
C+-----------------------------------------------------------------------------
C
C   -----------------------
C   T A U _ C A L _ O P E N
C   -----------------------
C
C   Description
C   -----------
C   Checks the existence of TAUCAL file and opens it if it
C   does in fact exist.
C
C
C   Parameters (">" Input, "<" Output, "W" Workspace, "!" Changed)
C
C   (>) REF_NAME:  (character) Reference name of TAUCAL file.
C   (>) PAR_NAME:  (character) Parameter name of TAUCAL file.
C   (<) TAU_EXIST: (logical) TRUE if TAUCAL file exists.
C   (!) STATUS:    (integer) Status flag
C
C
C   Prompts
C   -------
C   USE_TAU -- TRUE if the user wants to use TAUCAL file.
C
C   "PAR_NAME" -- Name of the TAUCAL structure 
C
C
C   External functions & subroutines called
C   ---------------------------------------
C   Library DSA:
C     DSA_NAMED_INPUT
C     DSA_WRFLUSH
C     DSA_WRUSER
C
C   Library ICH:
C     ICH_LEN
C
C   Library GEN:
C     GEN_EXIST
C
C   Library PAR:
C     PAR_BATCH
C     PAR_RDCHAR
C     PAR_RDKEY
C
C   Library TAU:
C     TAU_CAL_DTA_NAME
C
C
C   INCLUDE statements
C   ------------------
C   INCLUDE 'TAU_COMMON'
C                                                
C
C   Extensions to FORTRAN77
C   -----------------------
C   IMPLICIT NONE / Names > 6 characters / INCLUDE
C
C
C   SUN-specific statements
C   -----------------------
C   None.
C
C
C   Author/s
C   --------
C   Jim Lewis    RGO  (jrl@uk.ac.cam.ast.mail)
C
C
C   History
C   -------
C   29-JUN-1990   - Original subroutine
C   03-JAN-1992   - Included TAUCAL common block (JRL)
C   28-JAN-1994   - Modified for use with standalone UNIX FIGARO v4.0 (JRL)
C
C
C+-----------------------------------------------------------------------------
      implicit none
C
C     Parameters
C
      character ref_name*(*),par_name*(*)
      integer status
      logical tau_exist
C
C     Functions
C
      logical par_batch
      logical gen_exist
      integer ich_len
C
C     Local variables
C
      character tau_cal_filename*80,file1*80,file2*80
      logical tau_found
C
C     TAUCAL common block
C
      include 'TAU_COMMON'
C
C     Inherited status...
C
      if (status .ne. 0) go to 500
C
C     Find out if intend to use a TAUCAL file.  Don't ask if
C     you happen to be in batch
C
      tau_exist = .true.
      if (.not. par_batch()) then
         call par_rdkey('use_tau',.true.,tau_exist)
      end if
C
C     Try and find TAUCAL file...
C
      if (tau_exist) then
         call par_rdchar(par_name//'/nocheck',' ',tau_cal_filename)
         file1 = tau_cal_filename(:ich_len(tau_cal_filename))//'.dst'
         file2 = tau_cal_filename(:ich_len(tau_cal_filename))//'.sdf'
         tau_found = (gen_exist(file1) .or. gen_exist(file2))
C
C        If the TAUCAL file can't be found then return with bad status...
C
         if (.not. tau_found) then
            call dsa_wruser(' ')
            call dsa_wrflush
            call dsa_wruser('*****I can''t find your TAUCAL file*****')
            call dsa_wrflush
            call dsa_wruser(' ')
            call dsa_wrflush
            status = 1
            go to 500
         end if
C
C        If it does exist, then open the file
C
         call dsa_named_input(ref_name,tau_cal_filename,status)
         if (status .ne. 0) go to 500
C
C        Get DTA_ object name for the file...
C
         call tau_cal_dta_name(ref_name,tau_cal_filename,dta_name,
     :   lendta,status)
      end if
C
C     Exit
C 
  500 continue
      end
                            
