      subroutine tau_cal_setnum(ref_name,runnum,object,taucal,ndim,
     : dims,nitems,value,status)
C+-----------------------------------------------------------------------------
C
C   ---------------------------
C   T A U _ C A L _ S E T N U M
C   ---------------------------
C
C   Description
C   -----------
C   Writes a real variable to TAUCAL structure.
C
C
C   Parameters (">" Input, "<" Output, "W" Workspace, "!" Changed)
C
C   (>) REF_NAME:  (character) Reference name for TAUCAL file.
C   (>) RUNNUM:    (integer) Run number of the sub-structure where element 
C                  is to be written
C   (>) OBJECT:    (character) The element name of the variable to be written
C   (>) TAUCAL:    (logical) TRUE if TAUCAL file exist.
C   (>) NDIM:      (integer) Number of dimensions of object
C   (>) DIMS:      (integer array DIMS(NDIM)) Dimensions of object
C   (>) NITEMS:    (integer) Number of bytes to be read
C   (>) VALUE:     (real array VALUE(NITEMS)) Actual value(s) of the desired variable
C   (!) STATUS:    (integer) Status flag
C
C
C   Prompts
C   -------
C   None
C
C
C   External functions & subroutines called
C   ---------------------------------------
C   Library ICH:
C     ICH_CI
C     ICH_LEN
C
C   Library TAU:
C     TAU_SETNUM
C
C
C   INCLUDE statements
C   ------------------
C   INCLUDE 'TAU_COMMON'
C                                                
C
C   Extensions to FORTRAN77
C   -----------------------
C   IMPLICIT NONE / INCLUDE / Names > 6 characters
C
C
C   SUN-specific statements
C   -----------------------
C   None.
C
C
C   Author/s
C   --------
C   Jim Lewis    RGO  (jrl@uk.ac.cam.ast.mail)
C
C
C   History
C   -------
C   02-JUL-1990   - Original subroutine
C   03-JAN-1992   - Included TAUCAL common block (JRL)
C   06-JAN-1992   - Included call to TAU_SETNUM (JRL)
C   28-JAN-1994   - Modified for use with standalone UNIX FIGARO v4.0 (JRL)
C
C
C+-----------------------------------------------------------------------------
      implicit none
C
C     Parameters
C
      integer runnum,nitems,status,ndim
      integer dims(ndim)
      real value(nitems)
      character ref_name*(*),object*(*)
      logical taucal     
C
C     Functions
C
      character ich_ci*8
      integer ich_len
C
C     Local variables
C
      integer ldta,dta_status
      character number*8,dta_run*80
C
C     TAUCAL common block
C
      include 'TAU_COMMON'
C
C     Inherited status
C
      if (status .ne. 0) go to 500
C
C     If TAUCAL isn't being used, then don't write anything...
C
      dta_status = 0
      if (.not. taucal) go to 500
C
C     Construct name.  If this is from a run the append the run number.
C     If not then create the proper name from the object name
C
      if (runnum .ne. 0) then
         number = ich_ci(runnum)
         dta_run = dta_name(:lendta)//'.RUN'
     :   //number(:ich_len(number))
      else
         dta_run = dta_name
      end if
      ldta = ich_len(dta_run)
C
C     Now write it out
C
      call tau_setnum(dta_run,ldta,object,ndim,dims,nitems,value,
     : status)
C
C     Exit
C
  500 continue
      end
