      subroutine tau_converge(iter,ilam,alamda,chisq,ntriv,ntriv_min,
     : dchi_max,run,again)
C+-----------------------------------------------------------------------------
C     TAU_CONVERGE --- Routine to evaluated the convergence of the fit.
C
C     Parameters (">" Input, "<" Output, "W" Workspace, "!" Changed)
C
C     (>) ITER:      (integer) Number of iterations done so far
C     (>) ILAM:      (integer) Ratio of old_lamda/new_lamda
C     (>) ALAMDA:    (real) Most recent value of lamda (scaling factor in fitting routines)
C     (>) CHISQ:     (real) Value of chi-squared from most recent fit.
C     (>) NTRIV:     (integer) Number of trivial fractional changes to chi-squared
C     (>) NTRIV_MIN: (integer) Minimum number of trivial changes for convergence
C     (>) DCHI_MAX:  (real) Definition of trivial change in chi-squared
C     (<) RUN:       (logical) Flag for runaway solution
C     (<) AGAIN:     (logical) Instruction to do the fit again.
C     
C
C     AUTHOR:      Jim Lewis -- RGO
C     DATE:        20-NOV-1990
C     UPDATE:      23-JAN-1991 -- Slight changes in definition of 'runaway' 
C                                 (JRL)
C                  28-JAN-1994 -- Modified for use with standalone UNIX FIGARO
C                                 v4.0 (JRL)
C
C+-----------------------------------------------------------------------------
      implicit none
C
C     Parameters
C
      real chisq,alamda
      integer iter,ilam,ntriv
      logical again,run
C
C     Local variables
C
      integer ntriv_min
      real chisq_tabs(2),dchi,dchi_max
C
C     Decide if it has converged or not.  Look first to
C     see if lamda has run away...
C
      run = .false.
      if (abs(log10(abs(alamda))) .gt. 12.0) then
         again = .false.
         run = .true.
C
C     If this is the first iteration, then go back automatically.
C
      else if (iter .eq. 1) then
         again = .true.
         chisq_tabs(1) = chisq
C
C     If you have done at least 2 iterations, then find the fractional
C     difference between the most recent two.
C
      else if (iter .ge. 2) then
         chisq_tabs(2) = chisq
         dchi = abs(chisq_tabs(2) - chisq_tabs(1))/chisq
C
C        If the change was not 'trivial' or the scaling parameter
C        was increased (failed fit), then go back and do it again.
C
         if ((dchi .gt. dchi_max) .or. (ilam .lt. 1)) then
            again = .true.
            chisq_tabs(1) = chisq_tabs(2)
C
C        Otherwise count it...
C
         else
            ntriv = ntriv + 1
C
C           If the number of trivial changes equals the user input value, then
C           stop iterating.  Otherwise continue...
C
            if (ntriv .ge. ntriv_min) then
               again = .false.
            else
               again = .true.
            end if
         end if
      end if
      end
