      subroutine tau_deriv(nord,nunk,coef,x,y,sumx,sumy)
C+-----------------------------------------------------------------------------
C
C   -----------------
C   T A U _ D E R I V
C   -----------------
C
C   Description
C   -----------
C   Finds the first total derivative of 2-d polynomials
C
C
C   Parameters -  (">" input, "<" output, "W" Workspace, "!" Changed)
C
C   (>) NORD:     (Integer) Order of the fit
C   (>) NUNK:     (Integer) Maximum number of unknowns
C   (>) COEF:     (Real array COEF(NUNK)) Fit coefs
C   (>) X:        (Real) X pixel position where derivative is evaluated.
C   (>) Y:        (Real) Y pixel position where derivative is evaluated.
C   (<) SUMX:     (Real) Derivative with respect to X
C   (<) SUMY:     (Real) Derivative with respect to Y
C
C
C   Prompts
C   -------
C   None
C
C
C   External functions & subroutines called
C   ---------------------------------------
C   None
C
C
C   INCLUDE statements
C   ------------------
C   None
C                                                
C
C   Extensions to FORTRAN77
C   -----------------------
C   IMPLICIT NONE / END DO / Names > 6 characters
C
C
C   SUN-specific statements
C   -----------------------
C   None.
C
C
C   Author/s
C   --------
C   Jim Lewis    RGO  (jrl@uk.ac.cam.ast.mail)
C
C
C   History
C   -------
C   23-JAN-1990   - Original subroutine
C   17-MAR-1992   - Fixed bug which caused undefined exponentiation if
C                   x,y = 0. (JRL)
C
C
C+-----------------------------------------------------------------------------
      implicit none
C
C     Parameters
C
      integer nunk,nord
      real coef(nunk),x,y,sumx,sumy
C
C     Functions
C
      real expon
C
C     Local variables
C
      integer iterm,nt,j,k,ip1,ip2
      real c1,c2
C
C     Calculate the total derivative at X,Y
C
      sumx = 0.0
      sumy = 0.0
      iterm = 0
      do j = 0,nord
         nt = j + 1
         do k = 1,nt
            iterm = iterm + 1
            ip1 = nt - k
            ip2 = k - 1
            c1 = float(ip1)
            c2 = float(ip2)
            if (ip1 .ne. 0) sumx = sumx + 
     :      coef(iterm)*(c1*expon(x,ip1-1)*expon(y,ip2))
            if (ip2 .ne. 0) sumy = sumy + 
     :      coef(iterm)*(c2*expon(x,ip1)*expon(y,ip2-1))
         end do
      end do
      end


      real function expon(x,iy)
      implicit none
C
C     Parameters
C 
      integer iy
      real x
C
C     If it's iy = 0 then set it value to one.  Otherwise to exponentiation
C
      if (iy .eq. 0) then
         expon = 1.0
      else
         expon = x**iy
      end if
      end
