      subroutine tau_display(array,nx,ny,low,high,xaxis,yaxis,table,
     : badpix,spx2d,epx2d,sta2d,end2d,ximv,yimv,status)
C+-----------------------------------------------------------------------------
C
C   ---------------------
C   T A U _ D I S P L A Y
C   ---------------------
C
C   Description
C   -----------
C   Does a quickie 2-d display of a frame.
C
C
C   Parameters (">" Input, "<" Output, "W" Workspace, "!" Changed)
C
C   (>) ARRAY:     (real array ARRAY(NX,NY)) Input array to be plotted.
C   (>) NX:        (integer) X dimension of data.
C   (>) NY:        (integer) Y dimension of data.
C   (>) LOW:       (real) Minimum value to plot
C   (>) HIGH:      (real) Maximum value to plot
C   (>) XAXIS:     (real array XAXIS(NX)) X axis data values
C   (>) YAXIS:     (real array YAXIS(NY)) Y axis data values
C   (>) TABLE:     (character) LUT name
C   (>) BADPIX:    (logical) True of magic values exist in the data
C   (<) SPX2D:     (integer array SPX2D(2)) Starting pixel values for plot.
C   (<) EPX2D:     (integer array EPX2D(2)) Ending pixel values for plot.
C   (<) STA2D:     (real array STA2D(2)) Starting axis values for plot
C   (<) END2D:     (real array END2D(2)) Ending axis values for plot
C   (<) XIMV:      (real array XIMV(2)) Viewport X axis endpoints
C   (<) YIMV:      (real array YIMV(2)) Viewport Y axis endpoints
C   (!) STATUS:    (integer) Status variable
C
C
C   Prompts
C   -------
C   "Do you want to change them" -- Gives the user the option to change the
C                                   grey scale levels.
C   "Type low grey level" -- Low grey level as chosen by the user
C   "Type high grey level" -- High grey level as chosen by the user
C
C
C   External functions & subroutines called
C   ---------------------------------------
C   Library DSA:
C     DSA_FREE_WORKSPACE
C     DSA_GET_WORKSPACE
C     DSA_TYPESIZE
C     DSA_WRFLUSH
C     DSA_WRUSER
C
C   Library DYN:
C     DYN_ELEMENT
C
C   Library ICH:
C     ICH_LEN
C
C   Library PAR:
C     PAR_QNUM
C     PAR_QUEST
C
C   Library NDP:
C     NDP_IMAGE_INDEX
C     NDP_IMAGE_LUT
C     NDP_IMAGE_PLOT
C     NDP_IMAGE_VIEWPORT
C
C   PGPLOT:
C     PGASK
C     PGBEGIN
C     PGPAGE
C
C   Library VAR:
C     VAR_GETCHR
C
C
C   INCLUDE statements
C   ------------------
C   INCLUDE 'DYNAMIC_MEMORY'
C   INCLUDE 'NDP_NUMERIC_RANGES'
C                                                
C
C   Extensions to FORTRAN77
C   -----------------------
C   IMPLICIT NONE / INCLUDE / END DO / DO WHILE / Names > 6 characters
C
C
C   SUN-specific statements
C   -----------------------
C   None.
C
C
C   Author/s
C   --------
C   Jim Lewis    RGO  (jrl@uk.ac.cam.ast.mail)
C
C
C   History
C   -------
C   03-MAY-1990   - Original subroutine
C   25-JUL-1990   - Parameter list changed to include LUT (JRL)
C   03-SEP-1990   - Minor alterations to give the user more choice in
C                   how the plot is done. (JRL)
C   28-JAN-1992   - Modified to use new NDP plotting routines. (JRL)
C   28-JAN-1994   - Modified for use with standalone UNIX FIGARO v4.0 (JRL)
C
C
C+-----------------------------------------------------------------------------
      implicit none
C
C     Parameters
C
      integer nx,ny,spx2d(2),epx2d(2),status
      real array(nx,ny),low,high,xaxis(nx),yaxis(ny),sta2d(2),
     : end2d(2),ximv(2),yimv(2)
      character table*(*)
      logical badpix
C
C     Functions
C
      integer dsa_typesize
      integer dyn_element
      integer ich_len
      logical par_quest
      logical par_qnum
C
C     Local variables
C
      integer totspace,address,bytes_int,islot,iptr
      real mag,sqvp,oldlow,oldhigh
      character control*2,softdev*20,message*80
      logical init,ans,ans2
      data init /.true./
C
C     Dynamic memory
C
      include 'DYNAMIC_MEMORY'
C
C     Numeric ranges
C
      include 'NDP_NUMERIC_RANGES'
C
C     Set up boundries of map
C
      spx2d(1) = 1
      spx2d(2) = 1
      epx2d(1) = nx
      epx2d(2) = ny
C
C     Magnification and control variable
C
      mag = 1.0
      control = 'C '
C
C     Starting and ending points in axis values
C
      sta2d(1) = xaxis(1)
      sta2d(2) = yaxis(1)
      end2d(1) = xaxis(nx)
      end2d(2) = yaxis(ny)
C
C     Get current PGPLOT screen device on initial entry
C
      if (init) then
         call var_getchr('softdev',0,0,softdev,status)
         if (status .ne. 0) then
            call dsa_wruser
     :      ('No screen device selected -- use SOFT command')
            call dsa_wrflush
            go to 500
         end if
      end if
      call pgbegin(0,softdev,1,1)
      call pgask(.false.)
      call ndp_image_lut(table,status)
      if (status .ne. 0) go to 500
C
C     Compute viewport for image
C
      call ndp_image_viewport(spx2d,epx2d,mag,control,ximv,yimv,sqvp)
C
C     Get workspace for data preparation
C
      bytes_int = dsa_typesize('int',status)
      totspace = nx*ny*bytes_int
      call dsa_get_workspace(totspace,address,islot,status)
      iptr = dyn_element(address)
      if (status .ne. 0) go to 500
C
C     Prepare the data now
C
      call ndp_image_index(nx*ny,low,high,array,badpix,
     : dynamic_mem(iptr),status)
      if (status .ne. 0) go to 500
C
C     Plot it
C
      control = 'AR'
      call ndp_image_plot(dynamic_mem(iptr),nx,ny,spx2d,epx2d,sta2d,
     : end2d,high,low,' ',control,ximv,yimv)
C
C     Now let the user change the colour levels
C
      write(message,100) low,high
  100 format('Colour levels set at:',2f10.3)
      call dsa_wruser(message(:ich_len(message)))
      call dsa_wrflush
      ans = par_quest('Do you want to change them?',.false.)
      do while (ans)
         oldlow = low
         oldhigh = high
         ans2 = par_qnum('Type low grey level',min_float,max_float,
     :   oldlow,.true.,' ',low)
         ans2 = par_qnum('Type high grey level',min_float,max_float,
     :   oldhigh,.true.,' ',high)
         call pgpage
         if ((oldhigh .ne. high) .or. (oldlow .ne. low)) then
            call ndp_image_index(nx*ny,low,high,array,badpix,
     :      dynamic_mem(iptr),status)
         end if
         call ndp_image_plot(dynamic_mem(iptr),nx,ny,spx2d,epx2d,
     :   sta2d,end2d,high,low,' ',control,ximv,yimv)
         ans = par_quest('Do you want to change them again?',.false.)
      end do      
C
C     Tidy and exit
C
      call dsa_free_workspace(islot,status)      
  500 end
