      subroutine tau_setnum(dta_name,length,object,ndim,dims,nitems,
     : value,status)
C+-----------------------------------------------------------------------------
C
C   -------------------
C   T A U _ S E T N U M
C   -------------------
C
C   Description
C   -----------
C   Writes a real variable to a structure.
C
C
C   Parameters (">" Input, "<" Output, "W" Workspace, "!" Changed)
C
C   (>) DTA_NAME:  (character) DTA object name for environment.
C   (>) LENGTH:    (integer) Length of DTA_NAME
C   (>) OBJECT:    (character) The element name of the variable to be written
C   (>) NDIM:      (integer) Number of dimensions of object
C   (>) DIMS:      (integer array DIMS(NDIM)) Dimensions of object
C   (>) NITEMS:    (integer) Number of bytes to be read
C   (>) VALUE:     (real array VALUE(NITEMS)) Actual value(s) of the desired variable
C   (!) STATUS:    (integer) Status flag
C
C
C   Prompts
C   -------
C   None
C
C
C   External functions & subroutines called
C   ---------------------------------------
C   Library DSA:
C     DSA_WRFLUSH
C     DSA_WRUSER
C
C   Library DTA:
C     DTA_CRNAM
C     DTA_CRVAR
C     DTA_DLVAR
C     DTA_ERROR
C     DTA_WRVARF
C
C   Library ICH:
C     ICH_LEN
C
C
C   INCLUDE statements
C   ------------------
C   INCLUDE 'DTACODES'
C                                                
C
C   Extensions to FORTRAN77
C   -----------------------
C   IMPLICIT NONE / INCLUDE / Names > 6 characters
C
C
C   SUN-specific statements
C   -----------------------
C   None.
C
C
C   Author/s
C   --------
C   Jim Lewis    RGO  (jrl@uk.ac.cam.ast.mail)
C
C
C   History
C   -------
C   06-JAN-1992   - Original subroutine.
C   28-JAN-1994   - Modified for use with standalone UNIX FIGARO v4.0 (JRL)
C
C
C+-----------------------------------------------------------------------------
      implicit none
C
C     Parameters
C
      integer nitems,status,ndim,length
      integer dims(ndim)
      real value(nitems)
      character object*(*),dta_name*(*)
C
C     Functions
C
      integer ich_len
C
C     Local variables
C
      integer dta_status,mdim
      character name*80,message*80
C
C     DTA error codes
C
      include 'DTACODES'
C
C     Inherited status
C
      dta_status = 0
      if (status .ne. 0) go to 500
C
C     Now construct full name with object appended.
C 
      if ((ndim .eq. 1) .and. (dims(1) .eq. 1)) then
         mdim = 0
      else
         mdim = ndim
      end if
      call dta_crnam(dta_name(:length),object(:ich_len(object)),mdim,
     : dims,name,dta_status)
C
C     Create the object in TAUCAL
C
      call dta_crvar(name(:ich_len(name)),'float',dta_status)
C
C     If the object already exists and it's an array, then delete the
C     old object and recreate it.  This is necessary in order to avoid
C     problems with differing array sizes.
C
      if (dta_status .eq. dta_exist) then
         if (mdim .gt. 0) then
            dta_status = 0
            call dta_crnam(dta_name(:length),object(:ich_len(object)),
     :      0,1,name,dta_status)
            call dta_dlvar(name(:ich_len(name)),dta_status)
            call dta_crnam(dta_name(:length),object(:ich_len(object)),
     :      mdim,dims,name,dta_status)
            call dta_crvar(name(:ich_len(name)),'float',dta_status)
            if (dta_status .ne. 0) go to 500
C
C        If it's a single variable then don't worry about it, just
C        overwrite the damn thing...
C
         else
            dta_status = 0
         end if
      end if
C
C     Now write it out...
C
      call dta_crnam(dta_name(:length),object(:ich_len(object)),
     : 0,1,name,dta_status)
      call dta_wrvarf(name(:ich_len(name)),nitems,value,dta_status)
      if (dta_status .ne. 0) go to 500
C
C     Exit
C
  500 if (dta_status .ne. 0) then
         call dta_error(dta_status,message)
         call dsa_wruser(message(:ich_len(message)))
         call dsa_wrflush
         status = 1
      end if
      end
