
                Phase Calibration of TAURUS data
                --------------------------------

Definitions
-----------

In general, the datacube used to derive the phase map will
be the result of observing NL different calibration lines, each in
NI different interference orders. The simplest and most common case 
is NL=1, NI=2. The intensity maxima of these lines lie on a set
of NL*NO 2D surfaces. The surfaces are defined theoretically by the 
following equations, which give the Z value of the intensity maxima
at each (X,Y) coordinate:
    
Z(I,L,THETA)  = Z0(L) + (I+2.G/LAM) * FSR * (SEC(THETA)-1)        (1)

L       =   Index of emission line   L=1,2.. NL
I       =   Index of interference order  I=0,1,.. NI-1
Z       =   Position along spectral axis of the peak of emission
            line L in order I at spatial pixel (X,Y)
G       =   Etalon gap           
LAM     =   Observing wavelength 
FSR     =   Free spectral range on the optical axis
THETA   =   Off-axis angle at spatial pixel (X,Y)

Z, Z0 and FSR are negative for TAURUS 1, positive for TAURUS 2

Throughout the reduction process the Z coordinate of the datacube 
must always be converted from pixel number to microprocessor units 
before any processing is done. Similarly, the phase map and FSR
must always be in terms of microprocessor units rather than
pixel units.

In general, the value of THETA will be made a complex function of
(X,Y) by geometric distortion of the detector. We therefore have:

TAN (THETA(X,Y)) = Distance of each pixel from optical axis          (2)
                   ----------------------------------------
                               Camera focal length

   
                 = SQRT( (XUM(X,Y)-XUM(X0,Y0))**2 + (YUM(X,Y)-YUM(X0,Y0))**2 )
                   -----------------------------------------------------------
                                              F


X,Y      =   Spatial co-ordinates of each point in pixel units
X0,Y0    =   Spatial coordinates of optical axis
Z0       =   Position along spectral axis of the peak of emission
             line L in order 0 on the optical axis
F        =   Focal length of camera in micron

XUM(X,Y) =   X coordinates in micron of each spatial pixel (X,Y)
         =   X * XSEP + XOFF(X,Y)    
XSEP     =   Average separation in micron of pixels along X direction
XOFF(X,Y)=   Offset in micron along X direction from a regular grid      
YUM(X,Y) =   Y coordinates in micron of each spatial pixel (X,Y)
         =   Y * YSEP + YOFF(X,Y)    
YSEP     =   Average separation in micron of pixels along Y direction
YOFF(X,Y)=   Offset in micron along Y direction from a regular grid      

Note that X0,Y0 will not in general be integers. When getting the values
of XUM(X0,Y0) and YUM(X0,Y0) it will be necessary to do a linear interpolation
between the two adjacent pixels.

A number of programmes will have to know the free spectral range of the
data at each (X,Y) point. This is given by:

         FSR(X,Y) = FSR * ( SEC(THETA)-1 )             (3)

Overview of phase calibration
-----------------------------

The first problem is to parametrise the geometric distortion of
the detector. This is done by the lookup tables XUM(X,Y) and YUM(X,Y),
which give the position in microns of each pixel. Note that in 
general, the physical position along the X axis of a detector 
pixel depends on both X and Y. These functions can be defined 
using observations of an aperture mask consisting of an array 
of pinholes of known separation. The programme MASK will fit a 
set of Gaussians to the images of the pinholes, then fit a 2D 
cubic spline to these results in order to parameterise the detector 
distortion.

The next stage is to fit the theoretical surfaces defined above 
to the observed data. By carrying out a multiple component Gaussian 
fit or moments analysis at each spatial pixel in the observed datacube, 
we define the set of NS surfaces ZOBS(X,Y,S). 

We then carry out a least squares fit of the theoretical surfaces to 
the observed surfaces. Of the variables in the equations above, 
both LAM and F are known instrumental parameters. The functions XUM 
and YUM have been defined by the MASK programme. Therefore the least 
squares routine will have to fit for Z0(L), G, FSR, X0, Y0.  The value 
of NL should be user specified, since it depends on a knowledge of the 
arc lamp used. NI could also be user specified, though a really 
intelligent fitting programme would work it out for itself.

The fitting programme is made somewhat complex by the fact that the
function being fitted is multivalued. We are fitting NI*NL theoretical
surfaces to NS observed surfaces. I guess the only sensible way to do 
the least squares minimisation is to minimise the following function:

        SIGMA ( MIN (ZOBS(X,Y,S)-Z(X,Y,I,L))**2 ) )

Where SIGMA is the sum over all values of X,Y,S
      MIN   is the minimum over all values of I,L

I suspect this will only work well if we have resonable first
guesses for most of the parameters. First guesses for (X0,Y0)
could come from fitting rings using RING. FSR and G should be 
pretty stable, and first guesses could be obtained from previous 
observing runs, via an instrument database. A first guess at
the FSR could also come from the radius versus frame relationship,
using the Roden FITFSR routine. First guesses at Z0(L) 
could easily be obtained from the values of ZOBS(X0,Y0,L); Z0(1) 
would be the minimum over all S of ZOBS(X0,Y0,S), Z0(2) would be the 
next lowest value etc.

In addition to fitting for the parameters listed above, the least
squares fit would give the residuals between the fitted surface and
actual data at each value of X, Y and S. By averaging at each value of
X and Y over all values of S we obtain a value for the residual at
each (X,Y), to be called the 'residuals map'.

We now have values for all the parameters which define the theoretical
phase map, and we also have a residuals map. We can therefore
calculate the form of the phase map at the different wavelength 
and FSR of the actual observation.

Calibration file
----------------

Throughout the calibration process the results should be written
to a CALIBRATION FILE. The idea is that all information on the
state of the instrument is stored in one place. The file should
be structured in a logical way, which reflects the way in which
data is reduced. The structure should also be reasonably flexible, 
to allow for example for the calibration and observations cubes 
to have different detector formats, micro settings. I suggest 
this is an HDS file, since some of the calibration results
will be images which can then be displayed with the standard
image display software. The detailed format I propose below
will no doubt be modified in the light of experience !

The first group of parameters in the file are items common
to all observations

.TELESCOPE .NAME
           .STATION
           .SCALE          (in arcsec/mm)
           .FOCNUM         (e.g. 11 for WHT Cass)              
           .FOCLEN

.TAURUS    .NUMBER         (1 or 2)

.COLLIMATOR.FOCNUM
           .FOCLEN

.ETALON    .NAME
           .COATING
           .NOMGAP         (nominal gap)
           .NOMFSR         (nominal FSR in wavelength independent microprocessor
                           units:    Actual FSR = NOMFSR * wavelength)

.CAMERA    .FOCNUM         (e.g. 2 or 4 for TAURUS 2 on WHT)
           .FOCLEN

.DETECTOR  .NAME

The next group of items are related to particular observations: 
MATRIX MASKS, CAL CUBES, RINGS and OBSERVATIONS. There is
a sub-structure for each different observation. We allow for 
the possibility of there being more than one observation of each 
type, by having an index i associated with each sub-structure
(i=1,2,3....)

.MASKi     .HEADER   .NUMRUN          (Parameters defining matrix mask observation)
                     .DATE
                     .UT
           .DETECTOR .XDIM            (Results from fitting to matrix mask)
                     .YDIM
                     .XSEP            (Average separation in micron of pixels along X direction)
                     .YSEP            (Average separation in micron of pixels along Y direction)
                     .XOFF(XDIM,YDIM) (Offset in micron along X direction of each pixel from a regular grid)
                     .YOFF(XDIM,YDIM) (offset in micron along Y direction of each pixel from a regular grid)
                     (note that XOFF and YOFF will sometimes be absent, in 
                      which case they should be assumed equal to zero)

.CALi      .HEADER   .NUMRUN         (Parameters defining cal cube observation)
                     .DATE
                     .UT
           .FILTER   .LAMCEN         (wavelength of filter)
                     .LAMWID         (bandpass of filter)
                     .POSITION       (pupil or iris)
                     .TILT           (Tilt of filter in degrees)
                     .LAMEFF         (LAMCEN corrected for TILT)
           .FORMAT   .ZSTART         (Coordinate of 1st pixel along Z axis)
                     .ZSTEP          (Increment along Z axis)
                     .ZDIM           (Z dimension)
                     .NMASK          (Pointer to MASK structure defining X and Y axes)
           .RING                     (Results of ring fits)
                     .NFITS          (Number of fits)
                     .Z(NFITS)       (z-value of plane in micro values)
                     .R(NFITS)       (radius of ring)  
                     .XCEN(NFITS)    (X centre of ring)
                     .YCEN(NFITS)    (Y centre of ring)
                     .ELLFAC(NFITS)  (Ellipticity of ring)
           .PHFIT                      (Results of fitting phase surface)
                     .XCEN             (X centre of phase map)
                     .YCEN             (Y centre of phase map)
                     .FSR              (Free spectral range in micro values)
                     .GAP              (Etalon gap in um)
                     .NLINE            (number of lines)
                     .Z(NLINE)         (peak of each line in micro values)
                     .NORDER           (max number of interference orders)
                     .RESID(XDIM,YDIM) (Residuals map in micro values)

.RINGi     .HEADER   .NUMRUN         (Parameters defining ring observation)
                     .DATE
                     .UT
           .FILTER   .LAMCEN         (wavelength of filter)
                     .LAMWID         (bandpass of filter)
                     .POSITION       (pupil or iris)
                     .TILT           (Tilt of filter in degrees)
                     .LAMEFF         (LAMCEN corrected for TILT)
           .FORMAT   .ZSTART         (Coordinate of Z axis)
                     .NMASK          (Pointer to structure defining X and Y axes)
           .RING     .Z              (equal to .MICRO.START)
                     .R              (radius of ring)  
                     .XCEN           (X centre of ring)
                     .YCEN           (Y centre of ring)
                     .ELLFAC         (Ellipticity of ring)

.OBSi      .HEADER   .NUMRUN         (Parameters defining observation)
                     .DATE
                     .UT
           .FILTER   .LAMCEN         (wavelength of filter)
                     .LAMWID         (bandpass of filter)
                     .POSITION       (pupil or iris)
                     .TILT           (Tilt of filter in degrees)
                     .LAMEFF         (LAMCEN corrected for TILT)
           .FORMAT   .ZSTART         (Coordinate of 1st pixel along Z axis)
                     .ZSTEP          (Increment along Z axis)
                     .ZDIM           (Z dimension)
                     .NMASK          (Pointer to MASK structure defining X and Y axes)
           .PHMAP                      (Parameters to be used for calculating phase surface)
                     .XCEN             (X centre of phase map)
                     .YCEN             (Y centre of phase map)
                     .ZCEN             (Z value of phase map at XCEN,YCEN)
                     .FSR              (Free spectral range in micro values)
                     .GAP              (Etalon gap in um)
                     .RESID(XDIM,YDIM) (Residuals map in micro values)
            

Programmes for phase calibration
--------------------------------

A standard analysis would then use the following TAURUS programmes:

TED  

This programme is an editor for the calibration file. It will
need to provide different options for different stages of the
calibration process. Like all the calibration programmes, it
musst be capable of running in batch mode.

a) Initialisation of calibration file. The first stage of the 
calibration process is to write information common to the entire
run into the calibration file. The user will be prompted for one 
of the 5 standard TAURUS setups: INT+TAURUS 1, WHT+TAURUS 2+f/2 
camera,  WHT+TAURUS 2+f/4 camera, AAT+TAURUS 2. The programme 
should then be able to fill in from a database the following
values in the calibration file:
.TELESCOPE .NAME
           .STATION
           .SCALE          
           .FOCNUM         
           .FOCLEN
.TAURUS    .NUMBER
.COLLIMATOR.FOCNUM
           .FOCLEN
.CAMERA    .FOCNUM         
           .FOCLEN
.DETECTOR  .NAME
The user will then be shown a list of the etalons available, derived
from a database, and asked which was used. The programme should 
then be able to fill in the following values in the calibration file:
.ETALON    .NAME
           .COATING
           .NOMGAP            
           .NOMFSR

 The databases must be easy to update, or add new entries.

b) Add information on a particular file. The next stage is to write
information specific to each datafile to the calibration file.
The programme first asks whether the file is a MASK, CAL, RING or 
OBS, and sets up the correct HDS structure in the calibration file, 
incrementing i by 1. The programme then asks for the run number, 
date and time of the observation (plus any other header info ?). 
These are written to the .HEADER sub-structure. For CAL, RING
and OBS the user is then asked for the central wavelength, bandpass,
position and tilt (in degrees) of the filter. The programme then 
calculates the effective wavelength using the equation:
       LAMEFF=LAMCEN * (1- TILT*TILT/28954 )     for POSITION=IRIS
       LAMEFF=LAMCEN                             for POSITION=PUPIL
The programme can then write the .FILTER structure.
For CAL, RING and OBS the user is then asked the index i
of the MASK structure which defines the detector coordinate 
system and this is written to .FORMAT.NMASK. The user is
then asked for the values of ZSTART and, in the case of CAL
and OBS, for ZSTEP and ZDIM also. The programme should make sure
that ZSTART and ZSTEP are negative for TAURUS 1 data.

c) Setting up first guesses for fitting the theoretical phase surface.
This involves assigning initial values to:
     .CALi  .PHFIT   .XCEN             
                     .YCEN         
                     .FSR          
                     .GAP          
                     .NLINE        
                     .NORDER       

of the arrays .RING.XCEN(NFITS) and .RING.YCEN(NFITS). If no RING
sub-structure is present in the .CALi structure, the user should
be asked for another .CALi or .RINGi structure which can be used.
.PHFIT.FSR is set equal to .ETALON.NOMFSR * .FILTER.LAMEFF
.PHFIT.GAP is set equal to the nominal gap stored in .ETALON.GAP.
.PHFIT.NLINE and .PHFIT.NORDER are specified by the user.

d) Setting up parameters for calculating a phase surface. The
user should specify a cal file CALi, and the programme then
sets 
   .OBSi.PHMAP.XCEN             =     .CALi.PHMAP.XCEN
              .YCEN             =                .YCEN
              .GAP              =                .GAP
              .RESID(XDIM,YDIM) =                .RESID(XDIM,YDIM)
The user specifies the central phase for the phase surface,
and this is stored as .OBSi.PHMAP.ZCEN.  
The FSR scales as wavelength, so we finally set:
.OBSi.PHMAP.FSR  =  CALi.PHMAP.FSR * .OBSi.FILTER.LAMEFF / .CALi.FILTER.LAMEFF

e) There should be an option of setting any item in the TAURUS
structure to a given value, and also displaying the value of any 
item.

RING2D and RING3D

The data reduction package from Roden contains these programmes,
which fit ring patterns to RING observations and all planes
in a cal array respectively. These programmes should be adapted
to use FIGARO, and to write the results to the .RING structure
in the calibration file. Note that the values in .RING.Z should
be in microprocessor units, not pixel units. Note that these
programmes run entirely interactively. It would also be worth
updating the interactive RING programme to provide an alternative
for when automatic fitting fails ! 

FITFSR

Another programme from the Roden package, which fits to the
results from the ring fit to get an estimate of the FSR. This
programme should be interfaced to the TAURUS calibration file.

RAN

Programme to analyse results of ring fitting, in order to
derive shifts in the values of XCEN, YCEN and ZCEN. The programme 
would first need to know the index i of a CAL file, and the indices 
of any number of RING files. The programme would just plot out either 
of .CALi.RING.XCEN, .CALi.RING.YCEN or .CALi.RING.R as a function
of .CALi.RING.Z.  It would then mark with the number i the
result for each of the .RINGi files. The plot limits should be
user specified. The user can then measure offsets between the
original calibration file and the ring files, to be applied
to XCEN, YCEN, ZCEN. He can then do this with the calibration 
file editor. A more clever programme would also have an option 
to derive the offsets automatically.

CORRECT

This should correct the datacube by shifting each point in the spectral
dimension by an amount given by a the phase map. This should be either 
an analytic phase map, the residuals map stored in .PHMAP.RESID, a 2D 
image whose name is specified by the user, or any combination of these. 
The analytic phase map is given by a formula similar to equation 1 above:
  P(X,Y) = ZCEN + (2 * G/LAM) * FSR * (SEC(THETA)-1)
       ZCEN = .OBSi .PHMAP .ZCEN
          G =              .GAP
        FSR =              .FSR
        LAM =       .FILTER.LAMEFF
THETA is derived at each (X,Y) by equation 2 above, with X0 = .OBSi.PHMAP.XCEN 
and Y0 = .OBSi.PHMAP.YCEN. and F equal to .CAMERA.FOCLEN. The value of 
.OBSi.FORMAT.NMASK defines which MASK structure is to be used for XUM(X,Y) 
and YUM(X,Y). 

The programme assumes that the spectral dimension is the fastest
moving dimension of the datacube. The dimensions of a 2D phase map
input by the user must therefore match the slower 2 dimensions of 
the datacube.

The shift must take account of the periodicity along the spectral
axis. The free spectral range at each point is given by equation 3
above. 

Given the phase map P(X,Y) and free spectral range FSR(X,Y) we
therefore rebin the datacube DATA(X,Y,Z) according to the 
following equation:
          DATA(X,Y,Z) = DATA(X,Y,ZDASH)
    where ZDASH =  Z+P(X,Y)  Modulus FSR(X,Y) 
Unfortunately ZDASH will not in general be integer, so it is
necessary to interpolate between pixels. 

CALCMAP

This programme would use the same algorithm as CORRECT to calculate
the phase map and the FSR as a function of (X,Y). It would then 
write these out as 2D images. This programme would be important 
when it was intended to do the phase correction in the detector 
memory system.

FIT

This programme is the centre of the calibration process. It takes as 
input an HDS results file from MOMS or AUTOPILOT. It then fits the
theoeretical phase surface to the surfaces defined by the results
file, as described earlier. First guesses for parameters such as 
the gap and FSR should already have been set up in the .CALi.PHMAP 
structure. Fit then writes the optimised values of these parameters 
back to this structure, plus the residuals. The least squares fitting 
routine should have options either to do a weighted fit using the 
VARIANCE arrays stored in the results file, or to use uniform weighting.

MASK

This programme is used to define the position in micron of each 
detector pixel. The input datafile is for an observation made
using an aperture mask consisting of an array of pinholes. The 
user would input the separation of the pinholes, SEP, and an
angle by which the array of pinholes is rotated from the normal 
(X,Y) coordinate system. The separation of the pinhole images at 
the detector, SEPDET, is given by the equation:
      SEPDET = SEP * .CAMERA.FOCNUM / .COLLIMATOR.FOCNUM
The programme can then calculate, to within an arbitrary (X,Y)
offset, the coordinates in micron of each pinhole image on the
detector. The programme should then fit Gaussians to each pinhole
image in the data array, to get the (X,Y) coordinates of each pinhole
image. By fitting a cubic spline to these results, the programme
can define the coordinates of each (X,Y) pixel in micron. 
It can then define the average separation of pixels along
the X and Y directions, XSEP and YSEP, and the offsets from
a rectangular grid, XOFF(X,Y) and YUM(X,Y). Note that in some
cases the pinhole mask will not be useable over the entire field 
of view. For the pixels not covered the values of XOFF and YOFF
should be set to zero. 

Data reduction
--------------

The normal flow of data reduction would be as follows.

Run TED in modes a and b to put information in the calibration
file on both the overall setup and each datafile to be used.

Run MASK to define the coordinates in micron of each pixel
on the detector.

Run RING3D on each calibration cube and RING2D on each ring observation.

Run either AUTOPILOT or MOMS to get an HDS results file which
defines the observed phase surfaces.

Run TED in mode c to set up first guesses for fitting the
theoretical phase surface to each cal file.

Run FIT to fit the phase surfaces.

Run TED in mode d to set up the paramaters for the phase surface
for each observation.

Run RAN in order to check for shifts in XCEN, YCEN and ZCEN
between the calibration cube and the observation.

Run CORRECT to phase correct each observation.

For straightforward datasets it should be possible to turn this
whole process into a few large batch jobs. It is therefore important
that the programmes are structured so as to be easily run in
batch mode. 


Wavelength Calibration
----------------------
