            Functional specification of TAURUS package
            ------------------------------------------

General points
--------------

1)  We will use FIGARO routines to get parameters and access data. This
is not an ideal solution, but it is generally agreed that the advantages
outweigh the disadvantages. Most of the applications programmers I have
spoken to accept FIGARO as a de facto standard for this type of work.

2)  We will use GKS for line graphics. Probably in the form of PGPLOT.

3)  There are 3 programmes listed below which need image display: SPECTRUM,
CUBAN and MOVIE. Until the IDI interfaces appear, these should be aimed
at the IKON display device. All the IKON-specific calls should be put 
in a distinct subroutine library. There is a case for leaving these 
programmes towards the end of the project, in the hope that the IDI 
interfaces will appear soon ?? 

4)  Bad data. All programmes must support magic value blanking as a means 
of editing bad data.

5)  VARIANCE and QUALITY arrays. The question of whether we should support 
VARIANCE and QUALITY arrays is a major area of disagreement for a general
package. For TAURUS this would be an unacceptable overhead, whereas others 
(JCMT, X-ray) regard them as  essential. We need a solution which although 
optimised for TAURUS at the moment, does not preclude more general use later.
I suggest that all programmes read two keywords, USE_VARIANCE and USE_QUALITY. 
If these are false, then the programme shouldn't try to use the VARIANCE and 
QUALITY arrays. If the keywords are true, then either the programme should 
use the VARIANCE and  QUALITY arrays properly, or if it doesnt know how to 
do this a warning message should be output. I imagine that the latter would 
be the most common case !

6)  Sort order.  I believe that it is reasonable to expect a user to
keep track of the sort order, especially since they can examine the axis
labels. In order to guard against errors, the axis information should be
monitored to the user whenever a datacube is opened. More complex schemes
(e.g. the DIMENSION parameter, or examining axis labels) are extremely
TAURUS-specific (for example, for ASPECT data the spectral dimension is
number 2) and may be dangerous if they are not supported by all applications.

7)  Periodicity.  Many programmes will have to act differently for
TAURUS data, which is periodic along the spectral axis, as compared to 
other data. I suggest that the relevant progammes read a hidden keyword 
USE_FSR. If this is false, then the programme should assume the data is 
non-periodic. If the parameter is true, the programme should expect the
name of a 2D file which contains the free spectral range at each point
in the image. This 2D file will have been generated during the calibration
process. Note that USE_FSR will also be set to false in the early stages 
of TAURUS reduction, when the free spectral range has not yet been determined

8) All programmes should cope with data arrays of type either I*2 or R*4.
It may be acceptable to do this by doing the type conversion 'on the fly',
but for greater efficiency it would be better to call different subroutines
depending on the data type. We'll have to experiment.



Programmes already in FIGARO
----------------------------

Format conversion      BDF <--> FIGARO
FITS reading and writing
Format conversion      R*4 <--> I*2
Routines for analysing/displaying spectra and images

General Cube Manipulation
-------------------------

TAUTOFIG and FIGTOTAU                           
Format conversion between old TAURUS package and FIGARO and back again. 
A programme to do the former already exists.

TRANSPOSE                                       
Swap axes of datacube. This should be more general than the present
SORT and DESORT, and driven by an input parameter ORDER. For example,
ORDER='312' would map input axis 1 onto output axis 3
                      input axis 2 onto output axis 1
                      input axis 3 onto output axis 2
The programme SORT would correspond to ORDER='312'
            DESORT would correspond to ORDER='231'
The programme must rearrange the data array in as efficient a way
as possible. It may be necessary to call system service routines
to find the size of the working set, and such VAX-specific calls 
should be isolated in a separate subroutine library. There is a general
purpose transposition programme in AIPS, which might serve as a 
template for this TRANSPOSE. In addition to sorting the data array,
the axis information must be reordered correctly.

SUBSET                                 
Subsetting/Supersetting. Take a subset of a cube or form a new cube
in which a supplied one forms a subset. 

COMB33                                 
Arithmetic operations between two cubes.  +,-,/,*,max,min,logical operations. 
Some of this may already exist in FIGARO

COMB32                                 
Arithmetic operations between images and cubes. Same operations as above 
would be available.  the range of frames to be processed would be user 
defined. Such operations as flat-fielding, background subtraction etc 
would be handled in this way.

COMB31                                   
Arithmetic operations between spectra and cubes. Same operations as above 
would be available, the y,z subset region to be processed would be user defined.

COMB30                                   
Arithmetic operations between scalars and cubes. Same operations. 
Specified x,y,z region. Some of this may already exist in FIGARO

STATS
Statistics of a cube; min,max,mean,median,rms,no of valid points.
This may already exist in FIGARO

DUMMY
Create a dummy cube of specified dimensions and initialise
the contents to a user-specified value. Also store axis information.
A version of this already exists.

AXFLIP
General axis flip program. This would reverse the order of the data points
in a cube along a specified axis (1,2 or 3). This should probably work
in situ on the data. The AXIS information should also be flipped.

RESAMPLE
A general 3D rotation and scaling in all 3 axes, using a sensible
interpolation scheme. The programme should do it best to update
the axis information in a sensible manner, although in some
cases this might be difficult/impossible

STORE
Replace part of a cube with a specified sub-cube. 2 important special
cases are where the sub-cube is 1D (a spectrum) or 2D (an image).
The option should be available to either add in the sub-cube to
the existing data, or overwrite the existing data.

SMOOTH
Smooth a cube by convolution with another function. There should
be options for the smoothing function to be a 3D sinc function, Gausian,
or top hat. The FWHM of the smoothing function along each of the axes 
1,2 and 3 should be user specified. The size of the region along each of the
axes over which the smooth is carried out should also be user
specified. Note that if the size in the z direction is 1, this reduces
to a 2D smooth, and if the size in the y and z directions is 1, this
reduces to a 1D smooth.

MOVIE
Quick sequential display of planes from a cube, perhaps with speed/direction 
of movie controlled by trackerball/mouse. Doing this efficiently will almost 
certainly be hardware dependent, and should probably be aimed at the IKON. 

SPECTRUM
Spectrum extraction and summation over small user-defined sections of 
a datacube. This should operate in two modes, graphics and non-graphics. 
In the former mode, an image is displayed and cursor used to define regions.
There should be an option in the graphics mode to enter the coordinates
of the region manually. Spectra are then summed over the specified regions, 
displayed, and optionally stored as 1D spectra on disk. In non-graphics mode, 
the user inputs coordinates over which the spectra should be summed, and 
the result is written as a 1D spectrum on disk. As far as this programme is 
concerned, the fastest moving axis is regarded as the spectral axis. In 
graphics mode, the dimensions of the image displayed should be the same as 
the 2nd and 3rd dimensions of the datacube.

COMPACT
Cube compression by the addition of a selected range of planes
along a specified axis (1, 2 or 3) to form an output image. The
axis information of the output image should be sensible. i.e.
if we sum along axis 2 of the cube, the axis information for the 
output image should be equal to that for axes 1 and 3 of the input
cube. A preliminary version of this programme already exists.

EDIT
Use magic value blanking to edit all points within a specified
region of the cube whose values lie outside a specified range.

Spectral-line Data-Cube Analysis 
--------------------------------

EMISSION
Obtaining a quick estimate of the line emission intensity across the field.
This programme will assume that the fastest moving axis is the spectral axis.
The algorithm should be based on that used by AUTOPILOT for line identification,
though other algorithms might be thought about as alternative options.

MOMS
Moments analysis in the spectral direction at all spatial positions in a
cube. This programme will assume that the fastest moving axis is the
spectral axis. There will be an option to bin the data spatially.
The order of the moments to be calculated would be specified by the
user. There would be an option to look for more than one component.
This would be done by getting the user to specify a minimum separation
of the components. Then after finding the first component, the programme
would mask off the specified region around this component and look
for a second component, then mask off the region around this component
before looking for a third component, etc. This option is necessary so
that the programme can be used when fitting the phase map to a TAURUS
cube, since at some points in the cube the calibration line will appear more
than once, and there may even be more than one calibration line.
The results will be written to the HDS RESULTS file described below. There 
should be an option of writing an estimate of the variance to the results file,
based on the peak number of counts at each spatial position.

AUTOPILOT
This is a large 'semi-intelligent' programme from the old package and was 
written by Joss Bland. It fits reasonably simple spectral profiles (e.g. 
Gaussian emission line plus background) automatically to a datacube using 
a mask to define points to be processed and constrained fit parameters. 
Results would be written to the HDS RESULTS file described below.
This programme will assume that the fastest moving axis in the datacube is 
the spectral axis. The programme should be made more general, catering
for non-periodic data, as well as TAURUS data with a known free spectral range. 
At the moment assumes constant errors in the spectrum which is being fitted. 
A low priority job would be to implement an option of weighting the fit 
using the VARIANCE array. This might be better as a separate programme. We 
should look at Dave Axons LONGSLIT programme to see whether this automatic 
programme can be made more powerful (fitting more components, more general 
tolerances and bounds).

CUBAN
Interactive RESULTS file display and manipulation. The new version should 
have the ability to fit more complicated line profiles with multiple 
Gaussians, Cauchy functions, etc. The LONGSLIT programme may be useful
as a model for the fitting algorithms. The line graphics used to plot 
arrays of spectra and their fits on the screen should be configured in a 
flexible manner (i.e. the number of spectra plotted, scaling etc should be 
user controlled). The spectra and fits should be plotted with different 
pen numbers, hopefully producing different coloured lines on the ARGS and 
IKON (?). 

CROSS
Derive a velocity map from a datacube by cross-correlating a template
1D spectrum with each spectrum in a datacube. The resulots should
be written to the HDS results file described below.

PLOT
Plot grids of spectra on a GKS graphics device. This should produce 
elegant (publishable quality) results and be more flexible than the 
current version. The programme will assume that the fastest moving
axis is the spectral axis. The user should be able to specify the
central pixel, no of plots in each direction, spatial binning in 
each direction, axis labels and title.

HDS RESULTS file
----------------

One of the basic aims of datacube reduction is to compress the 
information in a 3D cube into a 2D image (velocity field, map of
emission line intensity etc.). AUTOPILOT and CUBAN do this by fitting
Gaussians to the spectrum at each point, MOMS does this by carrying
out a moments analysis of each point, CROSS by cross-correlating with
a template spectrum. We need a convenient way of
storing these results. The original versions of AUTOPILOT and
CUBAN wrote results to a datablock. This was a datacube with spatial
dimensions corresponding to the original datacube, but where each of 
the planes of the datacube contain a different parameter. For example,
one plane of the datablock would contain the peak intensity from the 
Gaussian fit, and another plane would contain the error. The main
problem with the datablock is that it is rather inflexible. If we create
a datablock big enough for a 1 component Gaussian fit, we have problems
if we later on decide we want to refit part of the cube with 2 Gaussians.
I therefore suggest that the fit parameters are stored as a series of
NDF images within one HDS container file. The error associated
with each parameter would not be stored as a separate image, but
in the VARIANCE component in the NDF. We would also need to store
mask images giving the number of components fitted at each point,
and the type of each component (Gaussian, moment etc.). How should all
these images fit together in the HDS container file ? I give one
suggestion below.,

I believe the most general structure is the 3 level hierarchy shown here

                   NCOMP       COMPONENTS
                                   |
  -----------------------------------------------------------------
  |                     |                 |                       |
                        |
               TYPE  PARAMETERS
                        |
       -------------------------------------------------
       |            |           |         |            |
     PAR1          PAR2        PAR3     PAR4         PAR5




At the top level is a mask image which gives the number of components 
fitted at each point NCOMP(x,y). The maximum value of this array is 
MAXCOMP. The structure element COMPONENTS contains MAXCOMP 
different structures. Inside each structure is a mask image TYPE(x,y) 
which gives the type of each component (Gaussian, Cauchy function, moment 
etc.). The value of TYPE determines the number of parameters required to 
define the fit (e.g. 4 for a single Gaussian; base, peak, centre, width). 
and the maximum value of this over the entire image is MAXPAR. The structure
element PARAMETERS contains MAXPAR different NDF images, each of which 
corresponds to one fit parameter.

The exact meaning of PAR1, PAR2 etc would depend on the type of fit
being performed. For a Gaussian fit PAR1 would be the base, PAR2
would be the height, PAR3 would be the position and PAR4 would
be the width. For a moments analysis PAR2 would be the zeroth moment,
PAR3 would be the 1st moment, PAR4 the second moment etc. This is
set up so the velocity and linewidth measured from Gaussian fitting
and from moments analysis go to a file with the same name.

This setup is very general, allowing any number of components at each
point, each component of a different type. However, this scheme is maybe 
overcomplicated ????????  It might also be inefficient in storage, if
simple line profiles were fitted at most points and more complex
profiles at one or two points... this would result in more structure
elements being created to accomodate the complex profiles. However,
these extra structure elements would be empty over most of the field,
and over these regions would have to be magic value blanked.


Astronomical Velocity Map Processing
------------------------------------

VELFIT
Programme to fit velocity maps with simple velocity fields;
rotating galaxies, rotation+outflow, warped disks etc. This
programme must be easy to modify so that people can try velocity
fields we haven't thought of. Much of this must already exist in
many packages. This programme is intended to form a template for
other 'astronomical' processing algorithms. 

Creation of datacubes
---------------------

FIBIN
Programme to turn data from a coherent fibre optic bundle into a 
spectral line datacube. The programme would be driven by a database,
which defines the spatial coordinates of the fibre corresponding to
each range of cross-sections in a wavelength-calibrated 2D longslit 
spectrum. The spectra would be extracted from the longslit spectrum 
in an optimum manner (??) and deposited in the datacube. Where spectra
are missing, perhaps due to a broken fibre, the corresponding points
in the datacube will be magic value blanked.

SLITIN
Programme to take a set of longslit spectra with the same wavelength
scale, but offset spatially from one another, into a datacube. 
The offset and PA of each slit would be user defined.

TAGIN
Programme to bin a set of photon events, tagged with an (X,Y,Z) coordinate
into a datacube.





