      subroutine rotation
      implicit none
C
C     Maximum number of rotation models catered for here
C
      integer max_mod
      parameter (max_mod=1)
C
C     Maximum number of coeffs for rotation model
C
      integer max_coef
      parameter (max_coef=11)
C
C     Local variables
C
      integer status,ndim,dims(10),nelm,nx,ny,vslot,address,vptr,
     : wptr,wslot,xptr,yptr,xslot,yslot,totspace,i,oslot,optr,
     : model,ifit,idum,unit,nord,ncoef,bytes_float,bytes_double,wptr2
      real incl,red,dummy,xc,yc,lamda0,rotcoefs(max_coef),maxradius
      logical badpix
      character type*10,modfile*80,output*80
C
C     Functions
C
      integer dsa_typesize
      integer dyn_element
      integer ich_fold
C
C     Numeric ranges
C
      include 'ndp_source:ndp_numeric_ranges.inc'
C
C     Magic values
C
      include 'ndp_source:ndp_magic_values.inc'
C
C     Dynamic memory
C
      include 'dyn_source:dynamic_memory.inc'
C
C     Common block for velocity field info
C
      common /stuff/ vptr,red,incl,nx,ny
C
C     Open DSA
C
      status = 0
      call dsa_open(status)
      if (status .ne. 0) go to 500
C
C     Get input velocity field file, and get its dimensions
C
      call dsa_input('vel_field','vel_field',status)
      call ndp_get_image_info('vel_field',.true.,.false.,type,badpix,
     : status)
      call dsa_data_size ('vel_field',2,ndim,dims,nelm,status)
      call dsa_use_flagged_values('vel_field',status)
      if (status.ne.0) goto 500
      if (ndim.ne.2) then
         call dsa_wruser('Data is not 2-dimensional\n')
         go to 500
      end if
      nx = dims(1)
      ny = dims(2)
      bytes_double = dsa_typesize('double',status)
      bytes_float = dsa_typesize('float',status)
      if (status .ne. 0) go to 500
C
C     Map the data
C
      call dsa_map_data('vel_field','read','float',address,vslot,status)
      vptr = dyn_element(address)
      if (status.ne.0) goto 500
C
C     Map the axis data arrays
C
      call dsa_map_axis_data('vel_field',1,'read','float',address,
     : xslot,status)
      xptr = dyn_element(address)
      call dsa_map_axis_data('vel_field',2,'read','float',address,
     : yslot,status)
      yptr = dyn_element(address)
      if (status .ne. 0) go to 500
C
C     Get inclination, redshift and rotation model to be used
C
      call par_rdval('inclin',min_float,max_float,1.0,'degrees',incl)      
      call par_rdval('redshift',min_float,max_float,1.0,'km/sec',red)
C
C     Do you want to fit or display a model?
C
      call dsa_wruser('Which do you want:\n')
      call dsa_wruser('(1)  Fit a new model\n')
      call dsa_wruser('(2)  Create residuals map from existing model\n')
      call par_rdval('fit',1.0,2.0,1.0,' ',dummy)
      ifit = nint(dummy)
C
C     If you are creating a residuals map, then read in the required parameters
C
      if (ifit .eq. 2) then
         call dsa_get_lu(unit,status)
         call par_rdchar('infile',' ',modfile)
         call fig_opfil2(modfile,' ',unit,' ',status)
         idum = ich_fold(modfile)
         if (status .ne. 0) then
            call dsa_wruser('Can''t find '//modfile//' -- ')
            call dsa_wruser('I can procede no further\n')
            call dsa_wruser(' \n')
            go to 500
         end if
         read(unit,*) 
         read(unit,*) xc,yc,lamda0
         read(unit,*) model
         if (model .eq. 1) then
            read(unit,*) nord
            ncoef = nord + 1
            read(unit,*) (rotcoefs(i),i=1,ncoef)
         end if
         read(unit,*) maxradius
         close(unit)
         call dsa_free_lu(unit,status)
         if (status .ne. 0) go to 500
C
C        Open a file for the output residuals map
C
         call par_rdchar('output',' ',output)
         call dsa_named_output('output',output,'vel_field',0,1,status)
         if (status .ne. 0) go to 500
C
C        Map the output file and fill it with magic values
C
         call dsa_map_data('output','write','float',address,oslot,
     :   status)
         if (status .ne. 0) go to 500
         optr = dyn_element(address)
         call gen_fill(nx*ny*bytes_float,magic_byte,dynamic_mem(optr))         
C
C        Get a small workspace
C
         totspace = (3 + ncoef)*bytes_double
         call dsa_get_workspace(totspace,address,wslot,status)
         if (status .ne. 0) go to 500
         wptr = dyn_element(address)         
C
C        Now create the residuals map...
C
         call cre_resid(dynamic_mem(vptr),nx,ny,red,incl,xc,yc,lamda0,
     :   rotcoefs,ncoef,model,maxradius,dynamic_mem(wptr),
     :   dynamic_mem(optr))
C
C     If you are doing a fit, then do it...
C
      else if (ifit .eq. 1) then
         call par_rdval('model',1.0,float(max_mod),1.0,' ',dummy)
         model = nint(dummy)
         if (model .eq. 1) then
            call par_rdval('nord',0.0,float(max_coef),1.0,' ',dummy)
            nord = nint(dummy)
            ncoef = nord + 1
         end if
C
C        Get some workspace
C
         totspace = nx*ny*bytes_float + (3 + ncoef)*bytes_double
         call dsa_get_workspace(totspace,address,wslot,status)
         if (status .ne. 0) go to 500
         wptr = dyn_element(address)      
         wptr2 = wptr + nx*ny*bytes_float
C
C        Now do the calculations
C
         call sub_rot(dynamic_mem(vptr),nx,ny,dynamic_mem(xptr),
     :   dynamic_mem(yptr),badpix,model,dynamic_mem(wptr),
     :   dynamic_mem(wptr2),ncoef,rotcoefs)
      end if
C
C     Tidy and exit
C
  500 call dsa_close(status)
      end


      subroutine cre_resid(v,nx,ny,red,incl,xc,yc,lamda0,rotcoefs,ncoef,
     : model,maxradius,xnag,output)
      implicit none
C
C     Parameters
C
      integer nx,ny,ncoef,model
      real v(nx,ny),red,incl,xc,yc,lamda0,rotcoefs(ncoef),output(nx,ny),
     : maxradius
      double precision xnag(3+ncoef)
C
C     Functions
C
      integer ich_len
C
C     Local variables
C
      integer i,j,nvar,xmin,xmax,ymin,ymax
      real sini,vv,resmin,resmax,res
      double precision dx,dy,theta,vmod
      character message*80
C
C     Numeric ranges
C
      include 'ndp_source:ndp_numeric_ranges.inc'
C
C     Magic values
C
      include 'ndp_source:ndp_magic_values.inc'
C
C     Common block with x,y ranges 
C
      common /ranges/ xmin,xmax,ymin,ymax
C
C     Set up the range in x,y for the evaluation routine
C
      xmin = max(1,nint(xc-maxradius))
      xmax = min(nx,nint(xc+maxradius))
      ymin = max(1,nint(yc-maxradius))
      ymax = min(ny,nint(yc+maxradius))
C
C     Now set up the model variables for the evaluation routine
C
      nvar = 3 + ncoef
      xnag(1) = dble(xc)
      xnag(2) = dble(yc)
      xnag(3) = dble(lamda0)
      do i = 1,ncoef
         xnag(3+i) = dble(rotcoefs(i))
      end do
C
C     Now evaluate the residuals...
C
      resmin = max_float
      resmax = min_float
      sini = sin(incl/57.2957795)
      do j = ymin,ymax
         dy = float(j) - xnag(2)
         do i = xmin,xmax
            dx = float(i) - xnag(1)
            vv = v(i,j)
            if ((vv .gt. magic_float) 
     :      .and. ((dx .ne. 0.0) .and. (dy .ne. 0.0))) then   
               call find_v(dx,dy,xnag,nvar,incl,theta,vmod)
               res = vv - red - sngl(vmod*cos(theta))*sini
               resmin = min(resmin,res)
               resmax = max(resmax,res)
               output(i,j) = res
            end if
         end do
      end do
      write(message,100) resmin,resmax
  100 format('Min and Max residuals are:',2f8.2,' km/sec\n')
      call dsa_wruser(message(:ich_len(message)))
      end


      subroutine sub_rot(v,nx,ny,xaxis,yaxis,badpix,model,work,
     : xnag,ncoef,rotcoefs)
      implicit none
C
C     Parameters
C
      integer nx,ny,ncoef,model
      real v(nx,ny),work(nx,ny),xaxis(nx),yaxis(ny),rotcoefs(ncoef)
      double precision xnag(3+ncoef)
      logical badpix
C
C     Functions
C
      integer dsa_typesize
      integer dyn_element
      logical par_batch
      logical par_qnum
C
C     Local variables
C
      integer status,ignore,stapix(2),endpix(2),nid,n,i,j,xmin,xmax,
     : ymin,ymax,ifail,totspace,address,wslot,npixels,nvar,unit,
     : lw,vptr,wpt1,wpt2,wpt3,wpt4,wpt5,nf,bytes_double,niter,iw
      real oldlow,oldhigh,low,high,start(2),end(2),ximv(2),yimv(2),
     : square,xc,yc,xlast,ylast,xr,yr,rcmax,x0,y0,dummy(2),lamda0
      double precision fsumsq
      character softdev*16,oldtable*80,table*80
      logical quit,plot
C
C     External routines
C
      external lsfun1,e04fdz
C
C     Dynamic memory
C
      include 'dyn_source:dynamic_memory.inc'
C
C     Magic values
C
      include 'ndp_source:ndp_magic_values.inc'
C                       
C     Numeric ranges
C
      include 'ndp_source:ndp_numeric_ranges.inc'
C
C     Common block with x,y ranges 
C
      common /ranges/ xmin,xmax,ymin,ymax
C
C     If this is a batch job, then get first guesses from command
C     line.  If not, then get them from either a 2-d plot or by hand
C
      if (par_batch()) then
         plot = .false.
      else
         call par_rdkey('plot',.true.,plot)
      end if
C
C     Section for getting some of the guesses from a plot
C
      if (plot) then
C
C        First get plot device for 2-d plot of the velocity info
C
         call var_getchr('softdev',0,0,softdev,status)
         if (status .ne. 0) then
            call dsa_wruser
     :      ('No screen device selected -- use SOFT command\n')
            go to 500
         end if
         call pgbegin(0,softdev,1,1)
C
C        Get LUT and min and max for display
C
         call var_getchr('table',0,0,oldtable,ignore)
         call par_qstr('Name of lookup table',oldtable,.true.,.true.,
     :   table)
         call var_setchr('table',0,0,table,ignore)
         call var_getnum('low',0,0,oldlow,ignore)
         call var_getnum('high',0,0,oldhigh,ignore)
         ignore = par_qnum('Low value for plot',min_float,max_float,
     :   oldlow,.true.,' ',low)
         ignore = par_qnum('High value for plot',min_float,max_float,
     :    oldhigh,.true.,' ',high)
         if (low .eq. high) then
            call dsa_wruser('Plot min and max can''t be the same!\n')
            go to 500
         end if
         call var_setnum('low',0,0,low,ignore)
         call var_setnum('high',0,0,high,ignore)
C
C        If there are magic values in the velocity data, then replace them
C
         if (badpix) then
            call replace_magic(v,nx,ny,low,high,work)
         end if
C        
C        Set up some axis info
C        
         stapix(1) = 1
         stapix(2) = 1
         endpix(1) = nx
         endpix(2) = ny
         start(1) = xaxis(1)
         start(2) = yaxis(1)
         end(1) = xaxis(nx)
         end(2) = yaxis(ny)
C        
C        Now do the plot
C        
         call ndp_image_lut(table,status)
         if (status .ne. 0) go to 500
         call ndp_image_viewport(stapix,endpix,1.0,'L',ximv,yimv,
     :   square)
         call ndp_image_plot(work,nx,ny,stapix,endpix,
     :   start,end,high,low,' ','AR',ximv,yimv)
C        
C        Locate the centre of the galaxy
C        
         call dsa_wruser('Indicate centre of galaxy\n')
         quit = .false.
         call ndp_image_cursor(xaxis,yaxis,nx,ny,start,end,
     :   stapix,endpix,1,'CDP',ximv,yimv,xc,yc,nid,xlast,
     :   ylast,quit)
         if (quit) go to 500
C        
C        Locate the zeropoint of azimuth
C
         call dsa_wruser('Indicate a point on the line of nodes\n')
         quit = .false.
         call ndp_image_cursor(xaxis,yaxis,nx,ny,start,end,
     :   stapix,endpix,1,'CDP',ximv,yimv,x0,y0,nid,xlast,
     :    ylast,quit)
         if (quit) go to 500
         lamda0 = atan2(y0-yc,x0-xc)
C        
C        Locate the point of maximum radius for calculation
C        
         call dsa_wruser('Indicate a point of maximum radius\n')
         quit = .false.
         call ndp_image_cursor(xaxis,yaxis,nx,ny,start,end,
     :   stapix,endpix,1,'CDP',ximv,yimv,xr,yr,nid,xlast,
     :   ylast,quit)
         if (quit) go to 500
         rcmax = sqrt((xr-xc)**2 + (yr-yc)**2)
C
C     Otherwise, get these things from the terminal
C
      else
         start(1) = xaxis(1)
         start(2) = yaxis(1)
         end(1) = xaxis(nx)
         end(2) = yaxis(ny)
         call ndp_par_rdary('centre',start,end,'A','pixels',2,2,dummy)
         xc = dummy(1)
         yc = dummy(2)
         call par_rdval('node_ang',-180.0,180.0,0.0,'degrees',lamda0)
         lamda0 = lamda0/57.2957795
         call par_rdval('radius',1.0,float(max(nx,ny)),1.0,'pixels',
     :   rcmax)
      end if   
C
C     Get initial guesses for rotation parameters
C
      call init_rot(model,ncoef,rotcoefs)
C
C     Get range in X and Y
C
      xmin = max(1,nint(xc-rcmax))
      xmax = min(nx,nint(xc+rcmax))
      ymin = max(1,nint(yc-rcmax))
      ymax = min(ny,nint(yc+rcmax))
C
C     Scale the coefs now so that everything is a bit closer to 1
C
      xnag(1) = dble(xc/float(nx))
      xnag(2) = dble(yc/float(nx))
      xnag(3) = dble(lamda0)
      xnag(4) = dble(0.01*rotcoefs(1))
      do i = 2,ncoef
         xnag(3+i) = dble(0.01*rotcoefs(i)*(float(nx)**float(i-1)))
      end do
C
C     Set up some variables
C
      npixels = 0
      do j = ymin,ymax
         do i = xmin,xmax
            if (v(i,j) .gt. magic_float) npixels = npixels + 1
         end do
      end do
      nvar = 3 + ncoef
C
C     Get some workspace
C
      bytes_double = dsa_typesize('double',status)
      lw = 6*nvar + npixels*nvar + 2*npixels + nvar*(nvar-1)/2
      totspace = (npixels + npixels*nvar + nvar + nvar**2 + lw)
     : *bytes_double
      call dsa_get_workspace(totspace,address,wslot,status)
      if (status .ne. 0) go to 500
      wpt1 = dyn_element(address)      
      wpt2 = wpt1 + npixels*bytes_double
      wpt3 = wpt2 + npixels*nvar*bytes_double
      wpt4 = wpt3 + nvar*bytes_double
      wpt5 = wpt4 + nvar*nvar*bytes_double
C
C     Do the fit
C
      ifail = 1
      call e04fcf(npixels,nvar,lsfun1,e04fdz,-1,400*nvar,0.5,1.0d0,
     : 1.0d5,xnag,fsumsq,dynamic_mem(wpt1),dynamic_mem(wpt2),npixels,
     : dynamic_mem(wpt3),dynamic_mem(wpt4),nvar,niter,nf,iw,1,
     : dynamic_mem(wpt5),lw,ifail)
C
C     Rescale now so that everything is back in sensible units...
C
      xc = sngl(xnag(1))*float(nx)
      yc = sngl(xnag(2))*float(nx)
      lamda0 = sngl(xnag(3))
      rotcoefs(1) = 100.0*sngl(xnag(4))
      do i = 2,ncoef
         rotcoefs(i) = 100*sngl(xnag(3+i))/(float(nx)**float(i-1))
      end do
      fsumsq = sqrt(fsumsq*1.0e4/float(npixels))
C
C     Write out results
C
      call dsa_get_lu(unit,status)
      if (status .ne. 0) go to 500
      open(unit=unit,file='rotation.out',status='new')
      write(unit,*) ' ifail =',ifail,'    rms =',sngl(fsumsq)
      write(unit,*) xc,yc,lamda0
      write(unit,*) model
      write(unit,*) ncoef-1
      write(unit,*) (rotcoefs(i),i=1,ncoef)
      write(unit,*) rcmax
      close(unit)
      call dsa_wruser('***Output to file ROTATION.OUT***\n')
  500 end
      

      subroutine init_rot(model,ncoef,rotcoefs)
      implicit none
C
C     Parameters
C
      integer model,ncoef
      real rotcoefs(ncoef)
C
C     Functions
C
      integer ich_len
C
C     Local variables
C
      character message*80
C                       
C     Numeric ranges
C
      include 'ndp_source:ndp_numeric_ranges.inc'
C
C     Send a message appropriate to the model chosen
C
      if (model .eq. 1) then
         call dsa_wruser('You have chosen a polynomial expansion of ')
         call dsa_wruser('the form\nTHETA = C0 + C1*R + C2*R**2...and ')
         write(message,100) ncoef-1
  100    format('of order',i3,'\n')
         call dsa_wruser(message(:ich_len(message)))
      end if
C
C     Now read the values...
C
      write(message,110) ncoef
  110 format('Please input estimates for',i3,' parameters\n')     
      call dsa_wruser(message(:ich_len(message)))
      call par_rdary('rotcoefs',min_float,max_float,'n',' ',ncoef,
     : ncoef,rotcoefs)
      end
      

      subroutine replace_magic(input,nx,ny,low,high,output)
C+--------------------------------------------------------------------------------------
C     REPLACE_MAGIC --- Temporarily replace magic values in 2-d array
C
C     Parameters (">" Input, "<" Output, "W" Workspace, "!" Changed)
C
C     (>) INPUT:       (real array INPUT(NX,NY)) Input data array
C     (>) NX:          (integer) X dimension of the data
C     (>) NY:          (integer) Y dimension of the data
C     (>) LOW:         (real) Lowest level for plotting
C     (>) HIGH:        (real) Highest level for plotting
C     (<) OUTPUT:      (real array OUTPUT(NX,NY)) Output data array
C
C
C     AUTHOR:      Jim Lewis -- RGO
C     DATE:        28-NOV-1990
C     UPDATE:
C
C+--------------------------------------------------------------------------------------
      implicit none
C
C     Parameters
C
      integer nx,ny
      real input(nx,ny),output(nx,ny),low,high
C
C     Local variables
C
      integer i,j
      real amin,data
C     
C     Magic values
C
      include 'ndp_source:ndp_magic_values.inc'
C
C     Find the lowest value to be plotted and subtract one
C
      amin = min(low,high) - 1.0
C
C     Now replace all magic values with this value
C
      do j = 1,ny
         do i = 1,nx
            data = input(i,j)
            if (data .gt. magic_float) then
               output(i,j) = data
            else
               output(i,j) = amin
            end if
         end do
      end do
      end


      subroutine lsfun1(iflag,npixels,nvar,xnag,fvecc,iw,liw,
     : w,lw)
      implicit none
C
C     Parameters
C
      integer npixels,nvar,iflag,liw,lw
      integer iw(liw)
      double precision xnag(nvar),fvecc(npixels),w(lw)
C
C     Functions
C
      real vel      
C
C     Local variables
C
      integer vptr,i,j,nx,ny,n,xmin,xmax,ymin,ymax
      double precision vval,sini,cosi,seci,pi,xscale,
     : dx,dy,phi,rho,rad,theta,vmod,pi2,reddble
      real red,incl,vv
C     
C     Magic values
C
      include 'ndp_source:ndp_magic_values.inc'
C
C     Dynamic memory
C
      include 'dyn_source:dynamic_memory.inc'
C
C     Common block for velocity field info
C
      common /stuff/ vptr,red,incl,nx,ny
C
C     Common block with x,y ranges 
C
      common /ranges/ xmin,xmax,ymin,ymax
C
C     Scan through map and calculate model residuals
C
      sini = sin(incl/57.2957795)
      xscale = 1.0/float(nx)
      reddble = dble(red)/100.0d0
      n = 0
      do j = ymin,ymax
         dy = xscale*float(j) - xnag(2)
         do i = xmin,xmax
            dx = xscale*float(i) - xnag(1)
            vv = vel(dynamic_mem(vptr),i,j,nx,ny)
            vval = dble(vv)
            if ((vv .gt. magic_float) 
     :      .and. ((dx .ne. 0.0) .and. (dy .ne. 0.0))) then   
               n = n + 1
               call find_v(dx,dy,xnag,nvar,incl,theta,vmod)
               fvecc(n) = reddble + vmod*cos(theta)*sini - vval/100.0d0
            end if
         end do
      end do
      end     


      subroutine find_v(dx,dy,xnag,nvar,incl,theta,vmod)
      implicit none
C
C     Parameters
C
      integer nvar
      real incl
      double precision vmod,theta,dx,dy,xnag(nvar)
C
C     Local variables
C
      double precision pi,pi2,phi,rho,rad,seci,cosi
C
C     Define pi
C
      pi = 4.0*atan(1.0)
      pi2 = 0.5*pi
      cosi = cos(incl/57.2957795)
      seci = 1.0/cosi
C
C     Now calculate theta and vmod
C
      phi = atan2(dy,dx) - xnag(3)
      if (phi .gt. pi) then
         phi = phi - 2.0*pi
      else if (phi .lt. -pi) then
         phi = phi + 2.0*pi
      end if
      rho = sqrt(dx*dx + dy*dy)
      rad = rho*sqrt(cos(phi)**2 + (sin(phi)*seci)**2)
      theta = atan(cosi*abs(tan(phi)))
      if ((phi .gt. pi2) .and. (phi .le. pi)) then
         theta = pi - theta
      else if ((phi .lt. 0.0) .and. (phi .ge. -pi2)) then
         theta = -theta
      else if (phi .lt. -pi2) then
         theta = -pi + theta
      end if
      call rotcrv(rad,xnag,nvar,vmod)
      end 


      subroutine rotcrv(r,xnag,npar,vmod)
      implicit none
C
C     Parameters
C
      integer npar
      double precision xnag(npar),r,vmod
C
C     Local variables
C
      integer i
      double precision sum
C
C     Calculate velocity as expansion
C
      sum = xnag(4)
      do i = 1,npar-4
         sum = sum + xnag(4+i)*r**float(i)
      end do
      vmod = sum
      end                  

      real function vel(v,i,j,nx,ny)
      implicit none
C
C     Parameters
C
      integer i,j,nx,ny
      real v(nx,ny)
C
C     Set the value...
C
      vel = v(i,j)
      end
